import os

from PyQt5.QtWidgets import *
from PyQt5.QtCore import QTimer
from mpipes.common_auth import *
from mpipes.common import *
import qdarkgraystyle
from requests.auth import *
import urllib3

from app.utils import *
from app.async import initAsync, getAsync
from app.horizontaltab import  TabWidget
from app.corrector import CorrectorMainWidget
from app.jobwidgets import JobsWidget
from app.borderlayout import *

os.environ['PYQTGRAPH_QT_LIB'] = 'PyQt5'  # Garantee pyqtgraph uses PyQt5 if avalilable
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)



class App(QMainWindow):

    def __init__(self):
        super().__init__()
        self.title = 'Parking desktop app'
        self.setWindowTitle(self.title)
        # drect = QDesktopWidget().availableGeometry()
        # drect.adjust(50,50,-50,-50)
        self.setGeometry(MAIN_WINDOW_SIZE)
        qt_rectangle = self.frameGeometry()
        center_point = QDesktopWidget().availableGeometry().center()
        qt_rectangle.moveCenter(center_point)
        self.move(qt_rectangle.topLeft())
        self.parking_app_widget = ParkingAppWidget(self)
        self.setCentralWidget(self.parking_app_widget)
        self.setContentsMargins(QMARGIN)
        self.show()


class ParkingAppWidget(QWidget):

    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.layout = BorderLayout(margin=MARGIN)

        self.auth_widget = TopWidget(self)
        self.layout.addWidget(self.auth_widget, BorderLayout.North)

        self.status_widget = QStatusBar(self)
        self.layout.addWidget(self.status_widget, BorderLayout.South)

        # Initialize tab screen

        icons = [
            os.path.join(this_dir,'img/webcam-control_record.png'),
            os.path.join(this_dir,'img/slideshow_report.png'),
            os.path.join(this_dir,'img/pictures-ok.png'),
            os.path.join(this_dir,'img/activity_monitor.png')
        ]
        self.tabs = TabWidget(icons=icons)
        self.tabs.setTabPosition(QTabWidget.West)
        # self.tabs.resize(300, 200)
        # self.dashboard_widget = QWidget(self)
        self.jobs_widget = JobsWidget(self)
        self.comm_logs = QTextEdit(self)
        self.comm_logs.setReadOnly(True)
        self.comm_logs.setLineWrapMode(QTextEdit.NoWrap)

        self.corrector_widget = CorrectorMainWidget(self)

        # Add tabs
        self.tabs.addTab(self.jobs_widget, "Jobs")
        # self.tabs.addTab(self.dashboard_widget, "Dashboard")
        self.tabs.addTab(self.corrector_widget,"Corrector")
        self.tabs.addTab(self.comm_logs, "Communication")

        # Add tabs to widget
        self.layout.addWidget(self.tabs, BorderLayout.Center)
        self.setLayout(self.layout)
        initAsync(self.status_widget, self.comm_logs)

    def status_message(self, string):
        self.status_widget.showMessage(string)

    def reloadUI(self):
        idx = self.tabs.currentIndex()
        if idx == 0:
            self.jobs_widget.reloadJobs()
        elif idx == 1:
            self.corrector_widget.reloadJobs()


    def getPrec(self):
        return self.auth_widget.getPrec()

    def getVals(self):
        return self.auth_widget.getVals()


class TopWidget(QWidget):
    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.layout = QHBoxLayout()
        self.layout.setContentsMargins(QMARGIN)
        title = QLabel('Authentication')
        self.layout.addWidget(title)
        self.auths = QComboBox()
        auths_ = self.list_auths()
        for auth in auths_:
            self.auths.addItem(auth)
        if len(auths_):
            self.auths.setCurrentText(get_default_authentication())
        self.auths.currentIndexChanged.connect(self.selectionchange)
        self.layout.addWidget(self.auths)
        self.layout.addStretch(1)
        self.selections = QWidget()
        slayout = QHBoxLayout()
        slayout.addStretch(1)
        slayout.addWidget(QLabel('Stats time range'))
        self.precs = QComboBox()
        for prec in ['20s', '10m', '1h', '1d']:
            self.precs.addItem(str(prec))
        self.precs.setCurrentText('10m')
        slayout.addWidget(self.precs)
        slayout.addWidget(QLabel('Stats values'))
        self.vals = QComboBox()
        for val in [1, 3, 5, 10, 20, 30, 40, 50, 60]:
            self.vals.addItem(str(val))
        self.vals.setCurrentText(str(10))
        slayout.addWidget(self.vals)
        self.selections.setLayout(slayout)
        self.layout.addWidget(self.selections)
        self.timer = QTimer()
        self.timer.setSingleShot(False)
        self.timer.start(10000)
        self.timer.timeout.connect(self.reload)
        self.layout.addWidget(QLabel('Update speed'))
        self.uspeed = QComboBox()
        for speed in [5, 10, 20, 30, 50, 60]:
            self.uspeed.addItem(str(speed))
        self.uspeed.setCurrentText('10')
        self.uspeed.currentIndexChanged.connect(self.speedselectionchange)
        self.layout.addWidget(self.uspeed)
        # self.layout.addWidget(QLabel('Style'))
        # self.styles = QComboBox()
        # for style in QStyleFactory.keys():
        #     self.styles.addItem(style)
        # self.styles.addItem('Qdarkstyle')
        # self.styles.currentIndexChanged.connect(self.styleselectionchange)
        # self.layout.addWidget(self.styles)
        self.setLayout(self.layout)

    def styleselectionchange(self, i):
        global app
        style = self.styles.currentText()
        if style == 'Qdarkstyle':
            app.setStyleSheet(qdarkgraystyle.load_stylesheet())
        else:
            app.setStyle(style)

    def speedselectionchange(self, i):
        spd = self.uspeed.currentText()
        spdi = 1000 * int(spd)
        self.timer.stop()
        self.timer.start(spdi)

    def selectionchange(self, i):
        auth = self.auths.currentText()
        self.parentWidget().status_message('Setting default authentication to {}'.format(auth))
        set_default_authentication(auth)
        async = getAsync()
        async.reloadCon()
        self.parent.reloadUI()


    def list_auths(self):
        basedir = base_dir()
        rt = []
        for f in os.listdir(basedir):
            if f.endswith(AUTH_FILE_SUFFIX):
                rt.append(os.path.splitext(f)[0])
        return rt

    def reload(self):
        # print('Reload')
        self.parentWidget().reloadUI()

    def getPrec(self):
        return self.precs.currentText()

    def getVals(self):
        return self.vals.currentText()



def main(argv=sys.argv[1:]):
    global app
    app = QApplication(sys.argv)
    app.setStyleSheet(qdarkgraystyle.load_stylesheet())
    app.setAttribute(Qt.AA_EnableHighDpiScaling, True)
    if hasattr(QStyleFactory, 'AA_UseHighDpiPixmaps'):
        app.setAttribute(Qt.AA_UseHighDpiPixmaps)
    # app.setStyle('Fusion')
    ex = App()
    sys.exit(app.exec_())



if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]))


import os
import json
import shutil


class ParkingDataset:

    def __init__(self, dir):
        self.add_allowed = True
        if not dir or not os.path.exists(dir):
            self.add_allowed = False
            return
            # raise Exception(f'Dir {dir} doesnt exists')
        self.src = os.path.join(dir, 'src')
        os.makedirs(self.src, exist_ok=True)


    def add_src_entry(self, job_id, regions_map, detections_map, img_src_file):
        if not self.add_allowed:
            return None, None
        jdir = os.path.join(self.src, self._version_base_dir(job_id, regions_map))
        dnamp_file = os.path.join(jdir, os.path.splitext(os.path.basename(img_src_file))[0] + '.json')
        im_file =  os.path.join(jdir, os.path.basename(img_src_file))
        with open(dnamp_file, mode='w') as fp:
            json.dump(detections_map, fp=fp, indent=4, sort_keys=True)
        shutil.copy2(img_src_file, im_file)
        return dnamp_file, im_file

    def _version_base_dir(self, job_id, regions_map):
        base_jid = self._single_version_check(job_id, regions_map)
        if base_jid:
            return base_jid
        for x in range(100000):
            base_jid = f'{job_id}-{x}'
            if self._single_version_check(base_jid, regions_map):
                return base_jid
        raise Exception('Not possible create or detect version of entry in {} for {}'.format(self.src, job_id))


    def _single_version_check(self, base_jid, regions_map):
        rmap_file = os.path.join(self.src, base_jid+'.json')
        if not os.path.exists(rmap_file):
            with open(rmap_file, mode='w') as fp:
                json.dump(regions_map, fp=fp, indent=4, sort_keys=True)
            os.makedirs(os.path.join(self.src, base_jid), exist_ok=True)
            return base_jid
        else:
            # compare
            regions_map_existing = None
            with open(rmap_file, mode='r') as fp:
                regions_map_existing = json.load(fp)
            if json.dumps(regions_map_existing, indent=4, sort_keys=True) == \
                    json.dumps(regions_map,indent=4, sort_keys=True):
                return base_jid
        return None


import os
import json
from datetime import timezone

from PyQt5 import QtGui
from PyQt5.QtCore import QTimer, Qt
from PyQt5.QtWidgets import QMessageBox, QWidget, QVBoxLayout, QTreeView, QListView, QListWidget, QListWidgetItem, \
    QGridLayout, QLabel, QLineEdit, QPushButton, QHBoxLayout, QCheckBox
from PyQt5.QtGui import QImage, QStandardItemModel, QStandardItem, QColor
import pyqtgraph as pg
import matplotlib.image as mpimg

from app.borderlayout import BorderLayout
from app.async import getAsync, AsyncComunicator
from app.utils import get_job_step_configuration, local_job_dir
from app.parking_dataset import ParkingDataset


class CorrectorMainWidget(QWidget):

    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        # Interpret image data as row-major instead of col-major
        self.parent = parent

        self.layout = BorderLayout(margin=5)

        self.job_treeview = QTreeView(self)
        self.job_treeview.setHeaderHidden(True)
        self.job_treeview.clicked.connect(self.job_selected)

        self.layout.addWidget(self.job_treeview, BorderLayout.West)

        self.job_widget = CorrectorJobWidget(self)
        self.layout.addWidget(self.job_widget, BorderLayout.Center)

        self.setLayout(self.layout)
        self.jobs = []

        self.defered = QTimer()
        self.defered.setSingleShot(True)
        self.defered.timeout.connect(self.reloadJobs)
        self.defered.start(500)

        self.job = None


    def job_selected(self, qmodelindex):
        if len(self.jobs) > 0:
            idx = self.job_treeview.currentIndex()
            item = self.model.itemFromIndex(idx)
            if not item.data():
                return
            self.job = item.data()
            self.job_widget.setJob(self.job)

    def reload_list(self):
        statuses = {}
        selected_item = None
        self.model = QStandardItemModel()
        rootNode = self.model.invisibleRootItem()
        for job in self.jobs:
            status = 'unknown'
            if 'status' in job:
                status = job['status']
            if status in statuses:
                stat_branch = statuses[status]
            else:
                stat_branch =  QStandardItem(status)
                rootNode.appendRow(stat_branch)
                statuses[status] = stat_branch
            childnode = QStandardItem(job['name'])
            childnode.setData(job)
            stat_branch.appendRow(childnode)
            if self.job and job['id'] == self.job['id']:
                selected_item = childnode

        self.job_treeview.setModel(self.model)
        for st in statuses:
            item = statuses[st]
            idx = self.model.indexFromItem(item)
            self.job_treeview.expand(idx)

        if selected_item:
            sidx = self.model.indexFromItem(selected_item)
            self.job_treeview.setCurrentIndex(sidx)


    def jobsLoaded(self, rt_items):
        if not rt_items:
            return
        self.jobs = rt_items
        self.reload_list()
        # self.job_edit_widget.updateJob()


    def reloadJobs(self):
        try:
            async = getAsync()
            async.list_jobs(self.jobsLoaded)
        except Exception as e:
            self.parent.status_message(str(e))

    def getVals(self):
        return self.parent.getVals()

    def getPrec(self):
        return self.parent.getPrec()


class CorrectorJobWidget(QWidget):

    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        # Interpret image data as row-major instead of col-major
        self.parent = parent

        self.layout = BorderLayout(margin=5)


        self.items_list = QListWidget()
        self.items_list.clicked.connect(self.job_detection_selected)
        self.layout.addWidget(self.items_list, BorderLayout.West)

        self.job_corrector = CorrectorRegions(self)
        self.layout.addWidget(self.job_corrector, BorderLayout.Center)

        self.setLayout(self.layout)
        self.job = None

    def setJob(self, job):
        self.job = job
        self.job_corrector.setJob(self.job)
        self.reloadList()


    def setJobFiles(self, items):
        self.items_list.clear()
        count = 0
        for item in items:
            if 'entry' in item and str(item['entry']).endswith('.json'):
                i = QListWidgetItem(os.path.splitext(item['entry'])[0])
                i.setData( Qt.UserRole, item)
                # if count > 20:
                #    i.setBackground(QColor(153,255,153))
                self.items_list.addItem(i)
                count += 1

    def reloadList(self):
        if not self.job:
            return
        async = getAsync()
        async.loadJobFiles(self.setJobFiles, self.job['id'])


    def job_detection_selected(self, idx):
        item  = self.items_list.currentItem()
        self.job_corrector.setDetection(item.text())


class CorrectorRegions(QWidget):

    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.parent = parent

        layout = BorderLayout()
        selected_region_widget = QWidget()
        selected_region_layout = QGridLayout()
        selected_region_layout.addWidget(QLabel('Selected area'), 1,0)
        self.selected_area_name = QLineEdit()
        selected_region_layout.addWidget(self.selected_area_name, 1,1)
        selected_region_layout.addWidget(QLabel('Occupied'), 2,0)
        self.regionOccupied = QCheckBox()
        selected_region_layout.addWidget(self.regionOccupied, 2,1)

        buttons = QWidget()
        correct_button = QPushButton("Correct")
        correct_button.clicked.connect(self.roiCorrect)
        save_regions_button = QPushButton('Save corrected regions')
        save_regions_button.clicked.connect(self.saveRegions)
        hbox = QHBoxLayout()
        hbox.addWidget(correct_button)
        hbox.addStretch(1)
        hbox.addWidget(save_regions_button)
        vbox = QVBoxLayout()
        vbox.addStretch(1)
        vbox.addLayout(hbox)
        buttons.setLayout(vbox)
        selected_region_layout.addWidget(buttons,3, 1)
        selected_region_widget.setLayout(selected_region_layout)
        layout.addWidget(selected_region_widget, BorderLayout.North)
        self.gv = pg.GraphicsLayoutWidget(parent=self)
        self.gv_view = self.gv.addPlot(lockAspect=True)
        self.gv_view.invertY()

        layout.addWidget(self.gv, BorderLayout.Center)
        self.setLayout(layout)
        self.blue_pen = pg.mkPen(color='b', width=3)
        self.red_pen = pg.mkPen(color='r', width=3)
        self.green_pen = pg.mkPen(color='g', width=3)
        self.rois = []
        self.rois_texts = []
        self.region_selected = None
        self.job = None
        self.detection_name = None
        self.corrected_regions = {}
        self.roi_selected = None
        self.parking_dataset = ParkingDataset('Z:\datasets\parking')



    def setJob(self, job):
        self.job = job
        self.detection_name = None
        cfg = get_job_step_configuration(self.job, 'parking_checker')
        self.gv_view.clear()
        self.rois.clear()
        self.rois_texts.clear()
        if cfg != None:
            self.ima = pg.ImageItem()
            self.gv_view.addItem(self.ima)
            if 'map' in cfg:
                jdir = local_job_dir(self.job['id'])
                self.used_img_file = os.path.join(jdir, cfg['map'])
                if not os.path.exists(self.used_img_file):
                    self.downloadUsedMap(cfg['map'])
                else:
                    self.setUsedMap(self.used_img_file)

            if not 'regions' in cfg:
                cfg['regions'] = []
            self.regions = cfg['regions']
            if not self.regions:
                return
            for item in self.regions:
                roi = pg.PolyLineROI([], pen=self.blue_pen, closed=True, removable=False)
                roi.setState(item['roi'])
                roi.setAcceptedMouseButtons(Qt.LeftButton)
                roi.sigClicked.connect(self.roiClicked)
                self.gv_view.addItem(roi)
                text = pg.TextItem()
                # text.setAnchor(-10)
                text.setText(str(item['name']), color='w')
                text.setParentItem(roi)
                rect = roi.boundingRect()
                text.setPos(rect.x(), rect.y())
                self.rois.append(roi)
                self.rois_texts.append(text)


    def getRegionStatus(self, region_name):
        for item in self.detections:
            if item['name'] == region_name:
                return str(item['occupancy']).lower() == 'true'
        return False

    def roiClicked(self, roi):
        # print(roi)
        idx = self.rois.index(roi)
        if self.region_selected and self.region_selected in self.regions:
            pidx = self.regions.index(self.region_selected)
            # self.rois_texts[pidx].setColor('w')
        self.region_selected = self.regions[idx]
        self.selected_area_name.setText(str(self.region_selected['name']))
        self.regionOccupied.setChecked(self.getRegionStatus(str(self.region_selected['name'])))
        self.roi_selected = roi
        # self.rois_texts[idx].setColor('y')

    def roiCorrect(self):
        if not self.region_selected or not self.region_selected in self.regions:
            return
        rname = str(self.region_selected['name'])
        occu = str(self.regionOccupied.isChecked()).lower()
        occuItem = {
            'name': rname,
            'occupancy': occu,
        }
        self.corrected_regions[rname] = occuItem
        pidx = self.regions.index(self.region_selected)
        self.rois_texts[pidx].setColor('y')
        if self.roi_selected:
            pen = self.blue_pen
            if occu == 'false':
                pen = self.green_pen
            else:
                pen = self.red_pen
            self.roi_selected.setPen(pen)
        # self.regions[idx]['name'] = self.selected_area_name.text()
        # self.rois_texts[idx].setText(self.selected_area_name.text(), color='w')


    def saveRegions(self):
        if not self.job or len(self.corrected_regions.keys()) == 0:
            return

        detf, imf = self.parking_dataset.add_src_entry(
            job_id=self.job['id'],
            regions_map=self.regions,
            detections_map=list(self.corrected_regions.values()),
            img_src_file=self.used_img_file)
        QMessageBox.information(self, "Corrections saved", f"Corrections saved to\n detections_file={detf}\n image_file={imf}",
                                  QMessageBox.Ok)

    def setUsedMap(self, ffname):
        if ffname and os.path.exists(ffname):
            self.used_img_file = ffname
            self.map_data = mpimg.imread(ffname)
            self.ima.setImage(self.map_data)

    def downloadUsedMap(self, mapfile):
        async = getAsync()
        async.download_used_map(self.setUsedMap, self.job['id'], mapfile)



    def detectionLoaded(self, ffname):
        if not ffname:
            return
        with open(ffname,'r') as fp:
            cfg_detection = json.load(fp)
        cfg = get_job_step_configuration(self.job, 'parking_checker')
        self.gv_view.clear()
        self.rois.clear()
        self.rois_texts.clear()
        self.corrected_regions = {}
        self.roi_selected = None
        if cfg != None:
            self.ima = pg.ImageItem()
            self.gv_view.addItem(self.ima)
            jdir = local_job_dir(self.job['id'])
            fname = '{}.jpg'.format(self.detection_name)
            self.used_img_file = os.path.join(jdir,fname)
            if not os.path.exists(self.used_img_file):
                async = getAsync()
                async.download_file( self.setUsedMap, self.job['id'],  'd',
                             fname)
            else:
                self.setUsedMap(self.used_img_file)

            if not 'regions' in cfg:
                cfg['regions'] = []
            self.regions = cfg['regions']
            if not self.regions:
                return
            self.detections = cfg_detection['detections']
            if not self.detections:
                return
            for x in range(len(self.regions)):
                item = self.regions[x]
                checked = self.detections[x]
                pen = self.blue_pen
                if 'occupancy' in checked and checked['occupancy'] == 'false':
                    pen = self.green_pen
                else:
                    pen = self.red_pen

                roi = pg.PolyLineROI([], pen=pen, closed=True, removable=False)
                roi.setState(item['roi'])
                roi.setAcceptedMouseButtons(Qt.LeftButton)
                roi.sigClicked.connect(self.roiClicked)
                self.gv_view.addItem(roi)
                text = pg.TextItem()
                # text.setAnchor(-10)
                text.setText(str(item['name']), color='w')
                text.setParentItem(roi)
                rect = roi.boundingRect()
                text.setPos(rect.x(), rect.y())
                self.rois.append(roi)
                self.rois_texts.append(text)


    def setDetection(self, detection_name):
        if not self.job:
            return
        self.detection_name = detection_name
        jdir = local_job_dir(self.job['id'])
        fname = '{}.json'.format(self.detection_name)
        ffname = os.path.join(jdir, fname)
        if not os.path.exists(ffname):
            async = getAsync()
            async.download_file(self.detectionLoaded, self.job['id'], 'd',
                                fname)
        else:
            self.detectionLoaded(ffname)





from PyQt5.QtCore import QRect, QSize, Qt
from PyQt5.QtWidgets import (QApplication, QFrame, QLabel, QLayout,
        QTextBrowser, QWidget, QWidgetItem)


class ItemWrapper(object):
    def __init__(self, i, p):
        self.item = i
        self.position = p

class BorderLayout(QLayout):
    West, North, South, East, Center = range(5)
    MinimumSize, SizeHint = range(2)

    def __init__(self, parent=None, margin=None, spacing=-1):
        super(BorderLayout, self).__init__(parent)

        if margin is not None:
            self.setContentsMargins(margin, margin, margin, margin)

        self.setSpacing(spacing)
        self.list = []

    def __del__(self):
        l = self.takeAt(0)
        while l is not None:
            l = self.takeAt(0)

    def addItem(self, item):
        self.add(item, self.West)

    def addWidget(self, widget, position):
        self.add(QWidgetItem(widget), position)

    def expandingDirections(self):
        return Qt.Horizontal | Qt.Vertical

    def hasHeightForWidth(self):
        return False

    def count(self):
        return len(self.list)

    def itemAt(self, index):
        if index < len(self.list):
            return self.list[index].item

        return None

    def minimumSize(self):
        return self.calculateSize(self.MinimumSize)

    def setGeometry(self, rect):
        center = None
        eastWidth = 0
        westWidth = 0
        northHeight = 0
        southHeight = 0
        centerHeight = 0

        super(BorderLayout, self).setGeometry(rect)

        for wrapper in self.list:
            item = wrapper.item
            position = wrapper.position

            if position == self.North:
                item.setGeometry(QRect(rect.x(), northHeight,
                        rect.width(), item.sizeHint().height()))

                northHeight += item.geometry().height() + self.spacing()

            elif position == self.South:
                item.setGeometry(QRect(item.geometry().x(),
                        item.geometry().y(), rect.width(),
                        item.sizeHint().height()))

                southHeight += item.geometry().height() + self.spacing()

                item.setGeometry(QRect(rect.x(),
                        rect.y() + rect.height() - southHeight + self.spacing(),
                        item.geometry().width(), item.geometry().height()))

            elif position == self.Center:
                center = wrapper

        centerHeight = rect.height() - northHeight - southHeight

        for wrapper in self.list:
            item = wrapper.item
            position = wrapper.position

            if position == self.West:
                item.setGeometry(QRect(rect.x() + westWidth,
                        northHeight, item.sizeHint().width(), centerHeight))

                westWidth += item.geometry().width() + self.spacing()

            elif position == self.East:
                item.setGeometry(QRect(item.geometry().x(),
                        item.geometry().y(), item.sizeHint().width(),
                        centerHeight))

                eastWidth += item.geometry().width() + self.spacing()

                item.setGeometry(QRect(rect.x() + rect.width() - eastWidth + self.spacing(),
                        northHeight, item.geometry().width(),
                        item.geometry().height()))

        if center:
            center.item.setGeometry(QRect(westWidth, northHeight,
                    rect.width() - eastWidth - westWidth, centerHeight))

    def sizeHint(self):
        return self.calculateSize(self.SizeHint)

    def takeAt(self, index):
        if index >= 0 and index < len(self.list):
            layoutStruct = self.list.pop(index)
            return layoutStruct.item

        return None

    def add(self, item, position):
        self.list.append(ItemWrapper(item, position))

    def calculateSize(self, sizeType):
        totalSize = QSize()

        for wrapper in self.list:
            position = wrapper.position
            itemSize = QSize()

            if sizeType == self.MinimumSize:
                itemSize = wrapper.item.minimumSize()
            else: # sizeType == self.SizeHint
                itemSize = wrapper.item.sizeHint()

            if position in (self.North, self.South, self.Center):
                totalSize.setHeight(totalSize.height() + itemSize.height())

            if position in (self.West, self.East, self.Center):
                totalSize.setWidth(totalSize.width() + itemSize.width())

        return totalSize


import json
import requests
from requests.auth import *
from _datetime import datetime
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *

import numpy as np
import cv2
import time
import traceback, sys
import mimetypes

from mpipes.common import AuthenticatedConn
from app.utils import local_job_dir, get_job_step_configuration_value

ASYNC = None

class WorkerSignals(QObject):
    '''
    Defines the signals available from a running worker thread.

    Supported signals are:

    finished
        No data

    error
        `tuple` (exctype, value, traceback.format_exc() )

    result
        `object` data returned from processing, anything

    progress
        `int` indicating % progress

    '''
    finished = pyqtSignal(str)
    error = pyqtSignal(tuple)
    result = pyqtSignal(object)
    progress = pyqtSignal(int)


class Worker(QRunnable):
    '''
    Worker thread

    Inherits from QRunnable to handler worker thread setup, signals and wrap-up.

    :param callback: The function callback to run on this worker thread. Supplied args and
                     kwargs will be passed through to the runner.
    :type callback: function
    :param args: Arguments to pass to the callback function
    :param kwargs: Keywords to pass to the callback function

    '''

    def __init__(self,  name, fn,  *args, **kwargs):
        super(Worker, self).__init__()

        # Store constructor arguments (re-used for processing)
        self.fn = fn
        self.name = name
        self.args = args
        self.kwargs = kwargs
        self.signals = WorkerSignals()

        # Add the callback to our kwargs
        # self.kwargs['progress_callback'] = self.signals.progress

    @pyqtSlot()
    def run(self):
        '''
        Initialise the runner function with passed args, kwargs.
        '''

        # Retrieve args/kwargs here; and fire processing using them
        try:
            result = self.fn(*self.args, **self.kwargs)
        except:
            # traceback.print_exc()
            exctype, value = sys.exc_info()[:2]
            self.signals.error.emit((self.name, exctype, value, traceback.format_exc()))
        else:
            self.signals.result.emit(result)  # Return the result of the processing
        finally:
            self.signals.finished.emit(self.name)  # Done


def initAsync( status_widget, log_window):
    global ASYNC
    ASYNC = AsyncComunicator(status_widget, log_window)

def getAsync():
    global ASYNC
    return ASYNC

class AsyncComunicator:

    def __init__(self, status_widget, log_window):
        self.status_widget = status_widget
        self.log_window = log_window
        self.threadpool = QThreadPool()
        self.con = None
        self.mime = mimetypes.MimeTypes()
        # print("Multithreading with maximum %d threads" % self.threadpool.maxThreadCount())

    def getCon(self):
        if not self.con:
            self.con = AuthenticatedConn()
        return self.con

    def reloadCon(self):
        self.con = AuthenticatedConn()

    def addLogText(self, text):
        if self.log_window:
            self.log_window.insertPlainText(text)
            self.log_window.verticalScrollBar().setValue(
            self.log_window.verticalScrollBar().maximum())

    def format_err_response(self, resp):
        status = "Error:"
        try:
            data = json.loads(resp.data.decode())
            status = "{} Http status {} error {}\n".format( str(datetime.now()), data['_status'], str(data['_error']).replace('\n',' '))
        except:
            status = "{} - {}\n".format(str(datetime.now()), resp.reason)
        self.addLogText(status)
        if self.status_widget:
            self.status_widget.showMessage(status)

    def start_info(self, name ):
        msg = "{} - {} - {}\n".format(str(datetime.now()),name, "request starting")
        self.addLogText(msg)
        if self.status_widget:
            self.status_widget.showMessage(msg)

    def finish_info(self, name):
        msg = "{} - {} - {}\n".format(str(datetime.now()), name, "request finished")
        self.addLogText(msg)
        if self.status_widget:
            self.status_widget.showMessage(msg)

    def error_info(self, err_tuple):
        name, exctype, value, formated = err_tuple
        msg = "{} - {} - {}\n".format(str(datetime.now()), name,  formated)
        self.addLogText(msg)
        if self.status_widget:
            self.status_widget.showMessage(msg)


    def communicate(self, name, execute_this_fn, result_fn,  *args, **kwargs):
        self.start_info(name)
        worker = Worker(name, execute_this_fn,  *args, **kwargs)  # Any other args, kwargs are passed to the run function
        worker.signals.result.connect(result_fn)
        worker.signals.finished.connect(self.finish_info)
        worker.signals.error.connect(self.error_info)
        # Execute
        self.threadpool.start(worker)


    def _list_jobs(self, offset=0, count=-1, page_size=10):
        con = self.getCon()
        rt_items = []
        if count != -1 and page_size > count:
            page_size = count
        while True:
            fileds = {
                'count': page_size,
                'offset': offset
            }
            resp = con.get_request('/api/micropipes/jobs/', fields=fileds)
            if resp.status != 200:
                self.format_err_response(resp)
                return None
            data = json.loads(resp.data.decode())
            items = data['_items']
            if len(items) == 0:
                return rt_items
            rt_items.extend(items)
            offset += len(items)
            if count != -1 and len(rt_items) + page_size > count:
                page_size = count - len(rt_items)
                if page_size == 0:
                    return rt_items

    def list_jobs(self,  result_fn):
        self.communicate("Load jobs",  self._list_jobs, result_fn)

    def _loadLastJobImage(self, job_id):
        con = self.getCon()
        fileds = {
            'from': '-3',
        }
        resp = con.get_request('/api/micropipes/jobs/{}/runtime/files/{}{}'.format(job_id,
                                                                                   'd', '/parking/'),
                               fields=fileds)
        if resp.status != 200:
            self.format_err_response(resp)
            return (None, None, None)
        data = json.loads(resp.data.decode())
        items = data['_items']
        if len(items) == 0:
            return (None, None, None)
        for item in reversed(items):
            if 'entry' in item and str(item['entry']).endswith('checked.jpg'):
                resp = con.get_request(
                    '/api/micropipes/jobs/{}/runtime/files/{}{}{}'.format(job_id,
                                                                          'd', '/parking/',
                                                                          item['entry']))
                if resp.status != 200:
                    self.format_err_response(resp)
                    return (None, None, None)
                arr = np.asarray(bytearray(resp.data), dtype=np.uint8)
                if (arr is None or arr.size == 0):
                    return (None, None, None)
                img = cv2.imdecode(arr, -1)
                return (img, item['entry'], item['time'])

    def loadLastJobImage(self, result_fn, job_id):
        self.communicate("Last job {} image".format(job_id), self._loadLastJobImage, result_fn, job_id)

    def _loadJobStats(self, job_id, prec, vals):
        con = self.getCon()
        fields = {
            'from': -1 * int(vals),
        }
        resp = con.get_request('/api/micropipes/jobs/{}/runtime/stats/{}'.format(job_id, prec),
                               fields=fields)
        if resp.status != 200:
            self.format_err_response(resp)
            return None
        data = json.loads(resp.data.decode())
        return data['_items']

    def loadJobStats(self, result_fn, job_id, prec, vals):
        self.communicate("Load job {} stats".format(job_id), self._loadJobStats, result_fn, job_id, prec, vals)


    def _loadJobLogs(self, job_id, level, vals):
        con = self.getCon()
        fields = {
            'start': -1 * int(vals),
        }
        resp = con.get_request('/api/micropipes/jobs/{}/runtime/logs/{}'.format(job_id, level),
                               fields=fields)
        if resp.status != 200:
            self.format_err_response(resp)
            return None
        data = json.loads(resp.data.decode())
        return data['_items']

    def loadJobLogs(self, result_fn, job_id, level, vals):
        self.communicate("Load job {} {} logs".format(job_id, level), self._loadJobLogs, result_fn, job_id, level, vals)


    def _jobStartStop(self, job_id, operation):
        con = self.getCon()
        resp = con.post_request('/api/micropipes/jobs/{}/{}'.format(job_id, operation), None)
        if resp.status != 200:
            self.format_err_response(resp)
            return False
        return True

    def jobStartStop(self, result_fn, job_id, operation):
        self.communicate("Job {} {} ".format(job_id, operation), self._jobStartStop, result_fn, job_id, operation)

    def _jobAdd(self, job_data):
        con = self.getCon()
        resp = con.post_request('/api/micropipes/jobs/', job_data)
        if resp.status != 201:
            self.format_err_response(resp)
            return False
        data = json.loads(resp.data.decode())
        job_id = data['_id']
        job = json.loads(job_data)
        temp_job_id = job['id']
        mapfile = get_job_step_configuration_value(job, 'parking_checker', 'map')
        if self._upload_if_not_exists(job_id, mapfile, temp_job_id=temp_job_id):
            return True
        return False

    def jobAdd(self, result_fn, job_data):
        self.communicate("Creating job", self._jobAdd, result_fn, job_data)

    def _jobEdit(self, job_id, job_data):
        con = self.getCon()
        stopEnsured = False
        retries = 0
        while not stopEnsured and retries < 10:
            self.addLogText('Get status of job {}\n'.format(job_id))
            resp = con.get_request('/api/micropipes/jobs/{}'.format(job_id))
            if resp.status != 200:
                self.format_err_response(resp)
                return False
            data = json.loads(resp.data.decode())
            ojob =  data['_items']
            if ojob['status'] == 'status.stop':
                stopEnsured = True
                continue
            if ojob['status']  == 'status.started':
                # stop it now
                self.addLogText('Will stop job {}\n'.format(job_id))
                resp = con.post_request('/api/micropipes/jobs/{}/stop'.format(job_id), None)
                if resp.status != 200:
                    self.format_err_response(resp)
                    return False
                time.sleep(1)
            retries += 1

        if not stopEnsured:
            self.addLogText('Failed to stop job {}\n'.format(job_id))
            return False
        self.addLogText('Sending job data {}\n'.format(job_id))
        resp = con.put_request('/api/micropipes/jobs/{}'.format(job_id), job_data)
        if resp.status != 200:
            self.format_err_response(resp)
            return False
        # background map fixture
        mapfile = get_job_step_configuration_value(json.loads(job_data), 'parking_checker', 'map')
        if self._upload_if_not_exists(job_id, mapfile):
            return True
        return False

    def jobEdit(self, result_fn, job_id, job_data):
        self.communicate("Edit job", self._jobEdit, result_fn, job_id, job_data)


    def _jobDelete(self, job_id):
        con = self.getCon()
        stopEnsured = False
        retries = 0
        while not stopEnsured and retries < 10:
            self.addLogText('Get status of job {}\n'.format(job_id))
            resp = con.get_request('/api/micropipes/jobs/{}'.format(job_id))
            if resp.status != 200:
                self.format_err_response(resp)
                return False
            data = json.loads(resp.data.decode())
            ojob =  data['_items']
            if ojob['status'] == 'status.stop':
                stopEnsured = True
                continue
            if ojob['status']  == 'status.started':
                # stop it now
                self.addLogText('Will stop job {}\n'.format(job_id))
                resp = con.post_request('/api/micropipes/jobs/{}/stop'.format(job_id), None)
                if resp.status != 200:
                    self.format_err_response(resp)
                    return False
                time.sleep(1)
            retries += 1

        if not stopEnsured:
            self.addLogText('Failed to stop job {}\n'.format(job_id))
            return False
        resp = con.delete_request('/api/micropipes/jobs/{}'.format(job_id))
        if resp.status != 200:
            self.format_err_response(resp)
            return False
        return True

    def jobDelete(self, result_fn, job_id):
        self.communicate("Delete job", self._jobDelete, result_fn, job_id)


    def _grab_data(self, img_url, img_url_user, img_url_pass ):
        self.addLogText('{} Connecting to {} as {} with {}\n'.format(str(datetime.now()), img_url, img_url_user, img_url_pass))
        response = requests.get(img_url, auth=HTTPDigestAuth(img_url_user, img_url_pass), verify=False)
        self.addLogText('{} Response status is {} - {}\n'.format(str(datetime.now()), response.status_code, response.reason))
        rt_n = None
        rt_len = 0
        if response.status_code == 200:
            btra = bytearray(response.content)
            rt_n = cv2.imdecode(np.asarray(btra), 1)
            rt_len = len(btra)
        self.addLogText('{} Read {} bytes\n'.format(str(datetime.now()), rt_len))
        return (rt_n, rt_len)

    def grabData(self, result_fn, img_url, img_url_user, img_url_pass):
        self.communicate("Grabbing image", self._grab_data, result_fn, img_url, img_url_user, img_url_pass)


    def _download_used_map(self, job_id, mapfile):
        jdir = local_job_dir(job_id)
        ffname = os.path.join(jdir, mapfile)
        con = self.getCon()
        resp = con.get_request(
            '/api/micropipes/jobs/{}/runtime/files/{}{}{}'.format(job_id,
                                                                  'p', '/parking/',
                                                                  mapfile))
        if resp.status != 200:
            self.format_err_response(resp)
            return None
        else:
            with open(ffname, 'wb') as f:
                f.write(resp.data)
            self.addLogText('Downloaded map {} for job {}\n'.format(mapfile, job_id))
            return ffname

    def download_used_map(self, result_fn, job_id, mapfile):
        self.communicate("Map {} for job {}".format(mapfile, job_id), self._download_used_map, result_fn, job_id, mapfile)

    def _upload_if_not_exists(self, job_id, mapfile, temp_job_id = None):
        if not mapfile or not job_id:
            return False
        con = self.getCon()
        fileds = {
            'from': '0',
        }
        resp = con.get_request('/api/micropipes/jobs/{}/runtime/files/{}{}'.format(job_id,
                                                                                   'p', '/parking/'),
                               fields=fileds)
        if resp.status != 200:
            self.format_err_response(resp)
            return False
        data = json.loads(resp.data.decode())
        items = data['_items']
        if len(items) > 0:
            for item in reversed(items):
                if 'entry' in item and str(item['entry']) == mapfile:
                    self.addLogText('Map {} for job {} already on server\n'.format(mapfile, job_id))
                    return True

        if temp_job_id:
            jdir = local_job_dir(temp_job_id)
        else:
            jdir = local_job_dir(job_id)
        ffname = os.path.join(jdir, mapfile)
        basename = mapfile
        with open(ffname, 'rb') as f:
            binary_data = f.read()
        mime_type = self.mime.guess_type(basename)[0]
        if not mime_type:
            mime_type = 'application/octet-stream'
        headers = {'content-type': mime_type}
        resp = con.post_request(
            '/api/micropipes/jobs/{}/runtime/files/{}{}{}'.format(job_id,
                                            'p','/parking/', basename),
                                        headerz=headers,
                                        payload=binary_data
                                )
        if resp.status != 201:
            self.format_err_response(resp)
            return False
        else:
            self.addLogText('Uploaded map {} for job {}\n'.format(mapfile, job_id))
            return True


    def _loadJobFiles(self, job_id):
        con = self.getCon()
        fileds = {
            'from': '0',
        }
        resp = con.get_request('/api/micropipes/jobs/{}/runtime/files/{}{}'.format(job_id,
                                                                                   'd', '/parking/'),
                               fields=fileds)
        if resp.status != 200:
            self.format_err_response(resp)
            return None
        data = json.loads(resp.data.decode())
        return data['_items']

    def loadJobFiles(self, result_fn, job_id):
        self.communicate("Job {} files".format(job_id), self._loadJobFiles, result_fn, job_id)

    def _download_file(self, job_id, duration, filename):
        jdir = local_job_dir(job_id)
        ffname = os.path.join(jdir, filename)
        con = self.getCon()
        resp = con.get_request(
            '/api/micropipes/jobs/{}/runtime/files/{}{}{}'.format(job_id,
                                                                  duration, '/parking/',
                                                                  filename))
        if resp.status != 200:
            self.format_err_response(resp)
            return None
        else:
            with open(ffname, 'wb') as f:
                f.write(resp.data)
            self.addLogText('Downloaded file {} for job {}\n'.format(filename, job_id))
            return ffname

    def download_file(self, result_fn, job_id,  duration, filename):
        self.communicate("Downloading {} with duration {} for job {}".format(filename, duration, job_id),
                         self._download_file, result_fn, job_id, duration, filename)


from datetime import datetime
import time
import pyqtgraph as pg
from PyQt5.QtCore import Qt

from PyQt5.QtWidgets import *
from pyqtgraph import PlotDataItem


class TimeAxisItem(pg.AxisItem):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setLabel(text='Time', units=None)
        self.enableAutoSIPrefix(False)

    def tickStrings(self, values, scale, spacing):
        return [datetime.fromtimestamp(value).strftime("%D %H:%M") for value in values]


class StatsWidget(QWidget):
    def __init__(self, parent, title):
        super(QWidget, self).__init__(parent)
        self.setMinimumHeight(250)
        self.layout = QVBoxLayout()
        self.graphWidget = pg.PlotWidget(parent=self, axisItems={'bottom': TimeAxisItem(orientation='bottom')},
                                         autoRange=True)
        self.graphWidget.setXRange(time.time(), time.time() + 100)
        # Add Background colour to white
        # Add Title
        self.graphWidget.setTitle(title)
        # Add Axis Labels
        # self.graphWidget.setLabel('left', 'Temperature (°C)', color='red', size=30)
        # self.graphWidget.setLabel('bottom', 'Hour (H)', color='red', size=30)
        # Add legend
        self.graphWidget.addLegend()
        # Add grid
        self.graphWidget.showGrid(x=True, y=True)
        # Set Range
        # self.graphWidget.setXRange(0, 100, padding=0)
        # self.graphWidget.setYRange(20, 555, padding=0)

        # self.plot(hour, temperature_1, "Sensor1", 'r')
        # self.plot(hour, temperature_2, "Sensor2", 'b')
        self.layout.addWidget(self.graphWidget)
        self.setLayout(self.layout)
        self.plotItems = {}

    def setXRange(self, x1, x2):
        self.graphWidget.setXRange(x1, x2)

    def setYRange(self, y1, y2):
        self.graphWidget.setYRange(y1, y2)

    def plot(self, x, y, plotname, color):
        if plotname in self.plotItems:
            pitem = self.plotItems[plotname]
            pitem.setData(x, y, connect="finite")
            self.graphWidget.replot()
        else:
            pen = pg.mkPen(color=color)
            pitem = PlotDataItem(x, y, name=plotname, pen=pen, symbol='o', symbolSize=3, symbolBrush=(color),
                                 connect="finite")
            self.plotItems[plotname] = pitem
            self.graphWidget.addItem(pitem, params={})

    def clear(self):
        for x in self.plotItems:
            item = self.plotItems[x]
            self.graphWidget.removeItem(item)
        self.plotItems.clear()

class NumberWidget(QWidget):
    def __init__(self, parent, title, main_op, ops, style):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.style = style
        # self.color = 'rgb(153,255,153)'
        self.setMaximumHeight(self.style['height'])
        self.setMinimumHeight(self.style['height'] - 100)
        self.setMaximumWidth(self.style['width'])
        self.setMinimumWidth(self.style['width'] - 100)
        layout = QHBoxLayout()

        main_num_widget = QWidget()
        main_num_layout = QVBoxLayout()
        label_title = QLabel()
        label_title.setText(title)
        label_title.setStyleSheet(self.style['subtitle'])
        main_num_layout.addWidget(label_title)
        self.label_main_op = QLabel()
        self.label_main_op.setStyleSheet(self.style['title'])
        main_num_layout.addWidget(self.label_main_op)
        self.label_subtitle = QLabel()
        main_num_layout.addWidget(self.label_subtitle)
        main_num_widget.setLayout(main_num_layout)
        layout.addWidget(main_num_widget)

        ops_num_widget = QWidget()
        ops_num_layout = QVBoxLayout()
        self.ops = {}
        for op in ops:
            label_title = QLabel()
            label_title.setText(op)
            label_title.setStyleSheet("font-size: 8pt;")
            ops_num_layout.addWidget(label_title)
            num_title = QLabel()
            num_title.setStyleSheet(self.style['numtitle'])
            ops_num_layout.addWidget(num_title)
            self.ops[op] = num_title

        ops_num_layout.addStretch(1)
        ops_num_widget.setLayout(ops_num_layout)
        layout.addWidget(ops_num_widget)
        layout.addStretch(1)
        layout.setAlignment(Qt.AlignTop)
        self.setLayout(layout)
        self.main_op = main_op
        self.title = title

    def getLastVal(self, stat):
        found = None
        for val in reversed(stat):
            if val:
                found = val
                break
        return found

    def clear(self):
        self.label_main_op.setText('')
        self.label_subtitle.setText('')
        for op in self.ops:
            op_w = self.ops[op]
            op_w.setText('')

    def setStat(self, stat, subtitle):
        if not stat or not self.main_op in stat:
            return

        main_val = self.getLastVal(stat[self.main_op])
        self.label_subtitle.setText("{}/{}".format(self.main_op, subtitle))
        if main_val:
            if 'multi' in self.style and self.style['multi']:
                main_val = main_val / self.style['multi']
            self.label_main_op.setText(self.style['num_format'].format(main_val))
        else:
            self.label_main_op.setText('')

        for op in self.ops:
            op_w = self.ops[op]
            op_val = self.getLastVal(stat[op])
            if op_val:
                op_w.setText(self.style['num_format'].format(op_val))
            else:
                op_w.setText('')



import os
import json
import time
import cv2
import numpy as np
from PyQt5 import QtGui
from PyQt5.QtGui import QImage
from PyQt5.QtWidgets import *
from PyQt5.QtCore import Qt
from PIL import Image
import tempfile

from app.utils import *
from app.borderlayout import *

class ImgFullView(QMainWindow):

    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.setWindowModality(Qt.ApplicationModal)
        self.title = "Image full scale view "
        self.setWindowTitle(self.title)
        # drect = QDesktopWidget().availableGeometry()
        # drect.adjust(SUBWINDOW_ADJUST,SUBWINDOW_ADJUST,-SUBWINDOW_ADJUST,-SUBWINDOW_ADJUST)
        self.setGeometry(SUB_WINDOW_SIZE)
        qtRectangle = self.frameGeometry()
        centerPoint = QDesktopWidget().availableGeometry().center()
        # centerPoint = mainWindow().geometry().center()
        qtRectangle.moveCenter(centerPoint)
        self.move(qtRectangle.topLeft())
        self.main_widget = QWidget()
        self.setCentralWidget(self.main_widget)
        layout = QHBoxLayout()
        scroll = QtGui.QScrollArea()
        self.ilabel = QLabel(self)
        self.ilabel.setAlignment(Qt.AlignCenter)
        scroll.setWidget(self.ilabel)
        scroll.setWidgetResizable(True)

        layout.addWidget(scroll)
        self.main_widget.setLayout(layout)

    def setPixmap(self, pixmap):
        self.ilabel.setPixmap(pixmap)


class JobImageWidget(QWidget):

    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.img_width = 640
        self.img_height = 480
        self.setMinimumHeight(420)
        # self.setMinimumWidth(900)
        self.layout = QVBoxLayout()
        self.name_label = QLabel()
        self.name_label.setStyleSheet('font-weight: bold; font-size: 16pt;')
        self.ilabel = QLabel(self)
        self.nlabel = QLabel(self)
        self.tlabel = QLabel(self)
        self.ilabel.resize(self.img_width, self.img_height)
        self.full_view = ImgFullView(self)

        self.clearImage()
        self.layout.addWidget(self.name_label)

        hlayout = QHBoxLayout()
        hwidget = QWidget()
        hlayout.setAlignment(Qt.AlignTop)
        hlayout.addWidget(self.ilabel)
        zoombutton = QPushButton()
        zoombutton.setIcon(QApplication.style().standardIcon(QStyle.SP_FileDialogContentsView))
        zoombutton.clicked.connect(self.show_zoomed)
        hlayout.addWidget(zoombutton)
        hwidget.setLayout(hlayout)


        self.layout.addWidget(hwidget)
        self.layout.addWidget(self.nlabel)
        self.layout.addWidget(self.tlabel)
        self.layout.setAlignment(Qt.AlignTop)
        self.layout.addStretch(1)
        self.setLayout(self.layout)
        self.src_pixmap = None
        # self.cv_img = None


    def clearImage(self):
        name = os.path.join(this_dir,'img','noimage.png')
        cvImg = cv2.imread(name)
        self.setImage(cvImg, name)

    def setImage(self, cvImg, name, time=''):
        # self.cv_img = cvImg.copy()
        cvImg = cv2.cvtColor(cvImg, cv2.COLOR_BGR2RGB)
        height, width, channel = cvImg.shape
        bytesPerLine = 3 * width
        qImg = QImage(cvImg.data, width, height, bytesPerLine, QImage.Format_RGB888)
        pixmap = QtGui.QPixmap.fromImage(qImg)
        self.src_pixmap = pixmap
        pixmap = pixmap.scaled(self.img_width, self.img_height, Qt.KeepAspectRatio, Qt.SmoothTransformation)
        self.ilabel.setPixmap(pixmap)
        self.nlabel.setText("{} {}x{} {}".format(name,width,height,channel))
        self.tlabel.setText(time)
        self.full_view.setPixmap(self.src_pixmap)

    def show_zoomed(self):
        if self.src_pixmap:
            self.full_view.setPixmap(self.src_pixmap)
            self.full_view.show()

    # def get_cv_img(self):
    #     try:
    #         tmpname = os.path.join(tempfile.gettempdir(),str(time.time())+'.jpg')
    #         cv2.imwrite(tmpname, self.cv_img)
    #         print('saved in {}'.format(tmpname))
    #         return tmpname
    #     except Exception as e:
    #         print(e)
    #         return None

class JobImageFilesDialog(QDialog):

    def __init__(self, parent, job_id):
        super(QDialog, self).__init__(parent)
        self.parent = parent
        self.setContentsMargins(QMARGIN)
        layout = BorderLayout(margin=MARGIN)
        info = QLabel('If this list is empty, do test in Camera Input configuration, to get some camera images.')
        layout.addWidget(info, BorderLayout.North)
        self.list = QListWidget()
        self.list.clicked.connect(self.file_selected)
        layout.addWidget(self.list, BorderLayout.West)
        self.img = JobImageWidget(self)
        layout.addWidget(self.img, BorderLayout.Center)
        button_ok = QPushButton('Use selected')
        button_ok.clicked.connect(self.close)
        layout.addWidget(button_ok, BorderLayout.South)
        self.setLayout(layout)
        self.job_id = job_id
        self.load_files()
        self.selected_file = None

    def load_files(self):
        dir = local_job_dir(self.job_id)
        files = os.listdir(dir)
        for f in files:
            self.list.addItem(f)

    def file_selected(self, qmodelindex):
        self.selected_file = self.list.currentItem().text()
        ffname = os.path.join(local_job_dir(self.job_id), self.selected_file)
        try:
            img_cv = cv2.imread(ffname)
            self.img.setImage(img_cv, self.selected_file, '')
        except:
            pass



from PyQt5.QtGui import QImage
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *


class HorizontalTabBar(QTabBar):

    def __init__(self, parent=None,  icons= None):
        QTabBar.__init__(self, parent)
        self.icons = icons

    def paintEvent(self, event):
        painter = QStylePainter(self)
        option = QStyleOptionTab()
        for index in range(self.count()):
            self.initStyleOption(option, index)
            painter.drawControl(QStyle.CE_TabBarTabShape, option)
            # painter.drawText(self.tabRect(index),
            #                 Qt.AlignCenter | Qt.TextDontClip,
            #                 self.tabText(index))
            if self.icons:
                rect = self.tabRect(index)
                rect.adjust(12,12,-12,-12)
                if index > len(self.icons):
                    continue
                img = QImage(self.icons[index])
                painter.drawImage(rect, img)


    def tabSizeHint(self, index):
        size = QTabBar.tabSizeHint(self, index)
        size.setHeight(64)
        size.setWidth(64)
        if size.width() < size.height():
            size.transpose()
        return size


class TabWidget(QTabWidget):
    def __init__(self, parent=None, icons=None):
        QTabWidget.__init__(self, parent)
        self.setTabBar(HorizontalTabBar(icons=icons))



from datetime import timezone

from PyQt5 import QtGui
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtGui import QStandardItemModel, QStandardItem
from mpipes.common import *

from app.borderlayout import BorderLayout
from app.configurationwidgets import *
from app.statswidgets import *

from app.async import getAsync


class RawView(QMainWindow):

    def __init__(self, parent):
        super().__init__()
        self.parent = parent
        self.setWindowModality(Qt.ApplicationModal)
        self.title = "Raw view - {}"
        self.setWindowTitle(self.title.format(''))
        # drect = QDesktopWidget().availableGeometry()
        # drect.adjust(SUBWINDOW_ADJUST,SUBWINDOW_ADJUST,-SUBWINDOW_ADJUST,-SUBWINDOW_ADJUST)
        self.setGeometry(SUB_WINDOW_SIZE)
        qtRectangle = self.frameGeometry()
        centerPoint = QDesktopWidget().availableGeometry().center()
        # centerPoint = mainWindow().geometry().center()
        qtRectangle.moveCenter(centerPoint)
        self.move(qtRectangle.topLeft())
        self.main_widget = QWidget()
        self.setCentralWidget(self.main_widget)
        self.setContentsMargins(QMARGIN)

        layout = BorderLayout()
        self.raw = QTextEdit(parent)
        self.raw.setReadOnly(True)
        self.raw.setLineWrapMode(QTextEdit.NoWrap)
        layout.addWidget(self.raw, BorderLayout.Center)
        self.main_widget.setLayout(layout)

    def setJob(self, job):
        self.raw.clear()
        if not job:
            return
        self.setWindowTitle(self.title.format(job['name']))
        self.raw.insertPlainText(json.dumps(job, indent=4, sort_keys=True))



class JobAttributesWidget(QWidget):

    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.main_layout = BorderLayout(margin=MARGIN)

        job_name_label = QLabel('Name')
        job_id_label = QLabel('ID')
        job_created_label = QLabel('Created')
        job_autostart_label = QLabel('Autostart')
        job_priority_label = QLabel('Priority')
        job_status_label = QLabel('Status')

        self.job_id_edit = QLabel()
        self.job_name_edit = QLineEdit()
        self.job_name_edit.textChanged.connect(self.changeName)
        self.job_status_edit = QLabel()
        self.job_created_edit = QLabel()
        self.job_autostart_edit = QCheckBox()
        self.job_priority_edit = QLineEdit()

        self.attribs_widget = QWidget(self)
        grid = QGridLayout()
        grid.setSpacing(MARGIN)
        etuples = [
            (job_name_label, self.job_name_edit),
            (job_priority_label, self.job_priority_edit),
            (job_autostart_label, self.job_autostart_edit),
            (job_id_label, self.job_id_edit),
            (job_created_label, self.job_created_edit),
            (job_status_label, self.job_status_edit),
        ]
        for x in range(len(etuples)):
            grid.addWidget(etuples[x][0], x + 1, 0)
            grid.addWidget(etuples[x][1], x + 1, 1)

        buttons = QWidget()
        view_button = QPushButton()
        view_button.setIcon(QApplication.style().standardIcon(QStyle.SP_FileDialogInfoView))
        view_button.clicked.connect(self.rawView)
        self.add_button = QPushButton("Clone")
        self.add_button.clicked.connect(self.cloneJob)
        self.edit_button = QPushButton("Save")
        self.edit_button.clicked.connect(self.saveJob)
        # edit_button.setIcon(QApplication.style().standardIcon(QStyle.SP_BrowserReload))
        self.stop_button = QPushButton("Stop")
        self.stop_button.clicked.connect(self.startStopJob)
        self.delete_button = QPushButton("Delete")
        self.delete_button.clicked.connect(self.deleteJob)
        hbox = QHBoxLayout()
        hbox.addWidget(view_button)
        hbox.addStretch(1)
        hbox.addWidget(self.add_button)
        hbox.addWidget(self.edit_button)
        hbox.addWidget(self.stop_button)
        hbox.addWidget(self.delete_button)
        vbox = QVBoxLayout()
        vbox.addStretch(1)
        vbox.addLayout(hbox)
        buttons.setLayout(vbox)
        grid.addWidget(buttons, len(etuples) + 1, 1)
        self.attribs_widget.setLayout(grid)
        self.main_layout.addWidget(self.attribs_widget, BorderLayout.North)
        self.setLayout(self.main_layout)
        self.job = None
        self.rawViewWindow = RawView(self)
        self.setButtonsState()

    def setJob(self, job):
        self.job = job
        if job:
            if 'id' in job:
                id = job['id']
                self.job_id_edit.setText(id)
            if 'status' in job:
                self.job_status_edit.setText(job['status'])
            else:
                self.job_status_edit.setText('')
            if 'name' in job:
                self.job_name_edit.setText(job['name'])
            else:
                self.job_name_edit.setText('')
            if 'created' in job:
                self.job_created_edit.setText(job['created'])
            else:
                self.job_created_edit.setText('')
            if 'priority' in job:
                self.job_priority_edit.setText(str(job['priority']))
            else:
                self.job_priority_edit.setText('0')
            if 'autostart' in job:
                self.job_autostart_edit.setChecked(job['autostart'])
            else:
                self.job_autostart_edit.setChecked(False)
        else:
            self.job_id_edit.setText('')
            self.job_name_edit.setText('')
            self.job_status_edit.setText('')
            self.job_created_edit.setText('')
            self.job_priority_edit.setText('')
            self.job_autostart_edit.setChecked(False)
        self.setButtonsState()

    def setButtonsState(self):
        if not self.job:
            self.add_button.setEnabled(False)
            self.edit_button.setEnabled(False)
            self.stop_button.setEnabled(False)
            self.delete_button.setEnabled(False)
            return
        if not 'created' in self.job:
            self.add_button.setEnabled(False)
            self.edit_button.setEnabled(True)
            self.stop_button.setEnabled(False)
            self.delete_button.setEnabled(False)
        else:
            self.add_button.setEnabled(True)
            self.edit_button.setEnabled(True)
            self.stop_button.setEnabled(True)
            self.delete_button.setEnabled(True)
        if 'status' in self.job:
            if 'created' in self.job:
                if self.job['status'] == 'status.started':
                    self.stop_button.setText('Stop')
                else:
                    self.stop_button.setText('Start')


    def changeName(self, name):
        if self.job:
            self.job['name'] = name

    def rawView(self):
        if not self.job:
            return
        self.saveValues()
        self.rawViewWindow.setJob(self.job)
        self.rawViewWindow.show()

    def saveValues(self):
        if not self.job:
            return
        self.job['name'] = self.job_name_edit.text()
        self.job['priority'] = int(self.job_priority_edit.text())
        self.job['autostart'] = self.job_autostart_edit.isChecked()


    def cloneJob(self):
        if self.job:
            self.saveValues()
            job = self.job.copy()
            job['id'] = track_uuid()
            if 'status' in job:
                job.pop('status')
            if 'created' in job:
                job.pop('created')
            self.parent.setJob( job )

    def jobOperationDone(self, success):
        if success:
            self.parent.reloadJobs()

    def jobDeleteDone(self, success):
        if success:
            self.parent.clearJob()
            self.parent.reloadJobs()

    def saveJob(self):
        if not self.job:
            return
        self.saveValues()
        async = getAsync()
        if  'created' in self.job:
            async.jobEdit(self.jobOperationDone, self.job['id'], json.dumps(self.job))
        else:
            async.jobAdd(self.jobOperationDone, json.dumps(self.job))


    def deleteJob(self):
        if not self.job:
            return
        ret = QMessageBox.warning(self, "Delete job warning", "Current job will be wiped from server and all data will be lost, are you sure?",
                                  QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
        if ret == QMessageBox.No:
            return
        async = getAsync()
        async.jobDelete(self.jobDeleteDone, self.job['id'])

    def startStopJob(self):
        if self.job and 'status' in self.job and 'created' in self.job:
            if self.job['status'] == 'status.started':
                # stop
                operation = 'stop'
            else:
                # start
                operation = 'start'

            async = getAsync()
            async.jobStartStop(self.jobOperationDone, self.job['id'], operation)


class JobStepWidget(QWidget):

    def __init__(self, parent, description, job_step_type, config_window):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.job_step_type = job_step_type
        layout = QHBoxLayout()
        label = QLabel(description)
        layout.addWidget(label)
        self.is_enabled = QCheckBox()
        self.is_enabled.clicked.connect(self.enabledChanged)
        layout.addWidget(self.is_enabled)
        self.config_button = QPushButton("Configure")
        self.config_button.clicked.connect(self.configure)
        layout.addWidget(self.config_button)
        self.setLayout(layout)
        self.job = None
        self.config_window = config_window

    def configure(self):
        if not self.job:
            return
        self.config_window.configure(self.job)
        self.config_window.show()

    def enabledChanged(self, enabled):
        if not self.job:
            return
        if enabled:
            self.config_button.setEnabled(True)
            self.addStep()
            self.config_window.configure(self.job)
        else:
            self.config_button.setEnabled(False)
            self.config_window.configure(self.job)
            self.config_window.removeConfiguration()
            self.removeStep()

    def removeStep(self):
        if not self.job:
            return
        if 'routes' in self.job and len(self.job['routes']) > 0:
            steps = self.job['routes'][0]['steps']
            found = None
            for step in steps:
                if 'worker_type' in step and step['worker_type'] == self.job_step_type:
                    found = step
                    break
            if found:
                steps.remove(found)

    def insertAfter(self, steps, precedesor, mystep):
        for x in range(len(steps)):
            step = steps[x]
            if 'worker_type' in step and step['worker_type'] == precedesor:
                steps.insert(x + 1, mystep)
                return True
        return False

    def addStep(self):
        if not self.job:
            return
        if 'routes' in self.job and len(self.job['routes']) > 0:
            steps = self.job['routes'][0]['steps']
            found = None
            for step in steps:
                if 'worker_type' in step and step['worker_type'] == self.job_step_type:
                    found = step
                    break
            if not found:
                mypos = STEPS_ORDER.index(self.job_step_type)
                precs = STEPS_ORDER[:mypos]
                mystep = {
                    'name': 'step_{}'.format(self.job_step_type),
                    'worker_type': self.job_step_type
                }
                inserted = False
                for precedesor in reversed(precs):
                    if self.insertAfter(steps, precedesor, mystep):
                        inserted = True
                        break
                if not inserted:
                    steps.insert(0, mystep)

    def setJob(self, job):
        self.job = job
        if not job:
            return
        if 'routes' in job and len(job['routes']) > 0:
            steps = job['routes'][0]['steps']
            for step in steps:
                if 'worker_type' in step and step['worker_type'] == self.job_step_type:
                    self.is_enabled.setChecked(True)
                    self.config_button.setEnabled(True)
                    return
        self.is_enabled.setChecked(False)
        self.config_button.setEnabled(False)


class JobStepsWidget(QWidget):
    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        layout = QVBoxLayout()
        layout.setContentsMargins(QMARGIN)
        self.steps = [
            JobStepWidget(self, 'Camera input', 'parking_image_input',
                          ConfigurationImageInput(self, 'parking_image_input')),
            JobStepWidget(self, 'Regions', 'parking_checker',
                          ConfigurationRegions(self, 'parking_checker')),
            JobStepWidget(self, 'Notification', 'notifier',
                          ConfigurationMqttNotifier(self, 'notifier')),
            JobStepWidget(self, 'Metrics', 'metrics_collector',
                          ConfigurationMetricsCollector(self, 'metrics_collector')),
            JobStepWidget(self, 'Archive', 'archiver',
                          ConfigurationArchive(self, 'archiver'))
        ]
        for x in range(len(self.steps)):
            layout.addWidget(self.steps[x])

        layout.addStretch(1)
        self.setLayout(layout)
        # self.temporaryImageMap = None

    def setJob(self, job):
        # self.temporaryImageMap = None
        for step in self.steps:
            step.setJob(job)


    # def setTemporaryImageMap(self, img):
    #     self.temporaryImageMap = img
    #
    # def getTemporaryImageMap(self):
    #     return self.temporaryImageMap

class JobWidget(QWidget):
    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.layout = QVBoxLayout()

        top = QWidget()
        top_layout = QHBoxLayout()
        self.image_widget = JobImageWidget(self)
        self.attributes_widget = JobAttributesWidget(self)
        self.overview_widget = JobOverviewWidget(self)
        self.steps_widget = JobStepsWidget(self)
        top_tabs = QTabWidget()
        top_tabs.addTab(self.overview_widget, 'Overview')
        top_tabs.addTab(self.attributes_widget, "Edit")
        top_tabs.addTab(self.steps_widget, "Steps")
        top_layout.addWidget(self.image_widget)
        top_layout.addWidget(top_tabs)
        top.setLayout(top_layout)

        self.layout.addWidget(top)

        # Initialize tab screen
        self.tabs = QTabWidget()
        self.tab_dashboard = JobStatsWidget(self)
        self.tab_files = QWidget()
        self.tab_logs = JobLogsWidget(self)
        # self.tabs.resize(300, 200)

        # Add tabs
        self.tabs.addTab(self.tab_dashboard, "Stats")
        self.tabs.addTab(self.tab_files, "Server files")
        self.tabs.addTab(self.tab_logs, "Server logs")

        # Add tabs to widget
        self.layout.addWidget(self.tabs, Qt.AlignTop)
        # self.layout.setAlignment(Qt.AlignTop)

        # self.logs_widget = JobLogsWidget(self)
        # self.layout.addWidget(self.logs_widget, BorderLayout.South)

        self.setLayout(self.layout)
        self.job = {}

    def setJob(self, job):
        self.initialState()
        self.job = job
        self.attributes_widget.setJob(job)
        self.steps_widget.setJob(job)
        self.tab_dashboard.setJob(job)
        self.overview_widget.setJob(job)
        self.tab_logs.setJob(job)

        self.loadLastJobImage()
        self.tab_dashboard.loadStats(self.parent.getPrec(), self.parent.getVals())
        self.overview_widget.loadStats(self.parent.getPrec(), self.parent.getVals())
        self.tab_logs.loadLogs(self.parent.getVals())

    def initialState(self):
        self.image_widget.clearImage()
        self.image_widget.name_label.setText('')

    def updateJob(self):
        if self.job and 'created' in self.job:
            self.loadLastJobImage()
            self.tab_dashboard.loadStats(self.parent.getPrec(), self.parent.getVals())
            self.overview_widget.loadStats(self.parent.getPrec(), self.parent.getVals())
            self.tab_logs.loadLogs(self.parent.getVals())

    def getJob(self):
        return self.job

    def reloadJobs(self):
        self.parent.reloadJobs()
        # self.parent.job_selected(0)

    def clearJob(self):
        self.setJob(None)
        self.parent.job = None

    def lastJobImageLoaded(self, rt_tuple):
        if not rt_tuple:
            return
        img, img_name, img_time = rt_tuple
        if not img_name:
            return
        self.image_widget.setImage(img, img_name, img_time)

    def loadLastJobImage(self):
        job = self.getJob()
        if not job or not 'created' in job:
            return
        self.image_widget.name_label.setText(job['name'])
        async = getAsync()
        async.loadLastJobImage( self.lastJobImageLoaded, job['id'])


class JobsWidget(QWidget):

    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        # Interpret image data as row-major instead of col-major
        self.parent = parent

        self.layout = BorderLayout(margin=5)
        # self.layout = QHBoxLayout(self)
        job_list = QWidget()
        job_list_layout = QVBoxLayout()

        self.job_treeview = QTreeView(self)
        self.job_treeview.setHeaderHidden(True)
        self.job_treeview.clicked.connect(self.job_selected)

        self.add_job_button = QPushButton('Add job')
        self.add_job_button.clicked.connect(self.add_job)
        job_list_layout.addWidget(self.add_job_button)
        job_list_layout.addWidget(self.job_treeview)
        job_list.setLayout(job_list_layout)
        self.layout.addWidget(job_list, BorderLayout.West)

        self.job_edit_widget = JobWidget(self)
        self.layout.addWidget(self.job_edit_widget, BorderLayout.Center)

        self.setLayout(self.layout)
        self.jobs = []
        self.con = None

        self.defered = QTimer()
        self.defered.setSingleShot(True)
        self.defered.timeout.connect(self.reloadJobs)
        self.defered.start(500)

        self.job = None


    def job_selected(self, qmodelindex):
        if len(self.jobs) > 0:
            idx = self.job_treeview.currentIndex()
            item = self.model.itemFromIndex(idx)
            if not item.data():
                return
            if self.job and not 'created' in self.job:
                ret = QMessageBox.warning(self, "Unsaved data warning", "Current job is unsaved, really select different one ?",QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
                if ret ==  QMessageBox.No:
                    return
            self.job = item.data()
            self.job_edit_widget.setJob(self.job)

    def reload_list(self):
        statuses = {}
        selected_item = None
        self.model = QStandardItemModel()
        rootNode = self.model.invisibleRootItem()
        for job in self.jobs:
            status = 'unknown'
            if 'status' in job:
                status = job['status']
            if status in statuses:
                stat_branch = statuses[status]
            else:
                stat_branch =  QStandardItem(status)
                rootNode.appendRow(stat_branch)
                statuses[status] = stat_branch
            childnode = QStandardItem(job['name'])
            childnode.setData(job)
            stat_branch.appendRow(childnode)
            if self.job and job['id'] == self.job['id']:
                selected_item = childnode

        self.job_treeview.setModel(self.model)
        for st in statuses:
            item = statuses[st]
            idx = self.model.indexFromItem(item)
            self.job_treeview.expand(idx)

        if selected_item:
            sidx = self.model.indexFromItem(selected_item)
            self.job_treeview.setCurrentIndex(sidx)


    def jobsLoaded(self, rt_items):
        if not rt_items:
            self.jobs = []
        else:
            self.jobs = rt_items
        self.reload_list()
        self.job_edit_widget.updateJob()


    def reloadJobs(self):
        try:
            async = getAsync()
            async.list_jobs(self.jobsLoaded)
        except Exception as e:
            self.parent.status_message(str(e))

    def getVals(self):
        return self.parent.getVals()

    def getPrec(self):
        return self.parent.getPrec()

    def add_job(self):
        self.job = new_job_template.copy()
        self.job['id'] = track_uuid()
        self.job_edit_widget.setJob( self.job )


class JobLogsWidget(QWidget):
    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.layout = BorderLayout()
        # Initialize tab screen
        self.tabs = QTabWidget()
        self.tab_debug = QTextEdit(self)
        self.tab_debug.setReadOnly(True)
        self.tab_debug.setLineWrapMode(QTextEdit.NoWrap)

        self.tab_info = QTextEdit(self)
        self.tab_info.setReadOnly(True)
        self.tab_info.setLineWrapMode(QTextEdit.NoWrap)

        self.tab_error = QTextEdit(self)
        self.tab_error.setReadOnly(True)
        self.tab_error.setLineWrapMode(QTextEdit.NoWrap)
        # self.tabs.resize(300, 200)

        # Add tabs
        self.tabs.addTab(self.tab_debug, "DEBUG")
        self.tabs.addTab(self.tab_info, "INFO")
        self.tabs.addTab(self.tab_error, "ERROR")

        # Add tabs to widget
        self.layout.addWidget(self.tabs, BorderLayout.Center)
        self.setLayout(self.layout)

        self.job = None


    def setJob(self, job):
        self.job = job
        self.initialState()

    def initialState(self):
        self.tab_debug.clear()
        self.tab_error.clear()
        self.tab_info.clear()

    def logsInfo(self, all):
        self.tab_info.clear()
        if not all:
            return
        LOCAL_TIMEZONE = datetime.now(timezone.utc).astimezone().tzinfo

        for m in all:
            t = str(datetime.strptime(
                m['time'], "%Y-%m-%dT%H:%M:%S.%f").replace(tzinfo=timezone.utc).astimezone(
                LOCAL_TIMEZONE))
            msg = "{} - {} - {}\n".format(t, m['worker_type'], m['message'])
            self.tab_info.insertPlainText(msg)
        self.tab_info.verticalScrollBar().setValue(
            self.tab_info.verticalScrollBar().maximum())

    def logsError(self, all):
        self.tab_error.clear()
        if not all:
            return
        LOCAL_TIMEZONE = datetime.now(timezone.utc).astimezone().tzinfo

        for m in all:
            t = str(datetime.strptime(
                m['time'], "%Y-%m-%dT%H:%M:%S.%f").replace(tzinfo=timezone.utc).astimezone(
                LOCAL_TIMEZONE))
            msg = "{} - {} - {}\n".format(t, m['worker_type'], m['message'])
            self.tab_error.insertPlainText(msg)
        self.tab_error.verticalScrollBar().setValue(
            self.tab_error.verticalScrollBar().maximum())

    def logsDebug(self, all):
        self.tab_debug.clear()
        if not all:
            return
        LOCAL_TIMEZONE = datetime.now(timezone.utc).astimezone().tzinfo

        for m in all:
            t = str(datetime.strptime(
                m['time'], "%Y-%m-%dT%H:%M:%S.%f").replace(tzinfo=timezone.utc).astimezone(
                LOCAL_TIMEZONE))
            msg = "{} - {} - {}\n".format(t, m['worker_type'], m['message'])
            self.tab_debug.insertPlainText(msg)
        self.tab_debug.verticalScrollBar().setValue(
            self.tab_debug.verticalScrollBar().maximum())

    def loadLogs(self, vals):
        if not self.job or not 'created' in self.job:
            return
        async = getAsync()
        async.loadJobLogs(self.logsInfo, self.job['id'], 'INFO', vals)
        async.loadJobLogs(self.logsError, self.job['id'], 'ERROR', vals)
        async.loadJobLogs(self.logsDebug, self.job['id'], 'DEBUG', vals)

class JobOverviewWidget(QWidget):
    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        layout = QVBoxLayout()

        stats_numbers_widget = QWidget()
        stats_numbers_layout = QHBoxLayout()
        self.number_stats = {
            'parking.checker.places_free': NumberWidget(self, 'Free places', 'avg', ['min', 'max'],
                                                        style={
                                                            'num_format': '{0:.0f}',
                                                            'title': 'font-weight: bold; color: rgb(153,255,153); font-size: 60pt;',
                                                            'subtitle': 'font-weight: bold; font-size: 20pt;',
                                                            'numtitle': 'font-weight: bold; color: rgb(153,255,153); font-size: 8pt;',
                                                            'width': 500,
                                                            'height': 250
                                                        }
                                                        ),
            'calculated.occupied': NumberWidget(self, 'Occupied', 'avg', [],
                                                style={
                                                    'num_format': '{0:.0f}',
                                                    'title': 'font-weight: bold; color: rgb(255,153,153); font-size: 60pt;',
                                                    'subtitle': 'font-weight: bold; font-size: 20pt;',
                                                    'numtitle': 'font-weight: bold; color: rgb(255,153,153); font-size: 8pt;',
                                                    'width': 500,
                                                    'height': 250
                                                }
                                                ),
        }
        for x in self.number_stats:
            stats_numbers_layout.addWidget(self.number_stats[x])

        stats_numbers_layout.addStretch(1)
        stats_numbers_widget.setLayout(stats_numbers_layout)
        layout.addWidget(stats_numbers_widget)

        small_stats_numbers_widget = QWidget()
        small_stats_numbers_layout = QHBoxLayout()
        small_number_stats = {
            'collector.job.logs.INFO': NumberWidget(self, 'Infos', 'count', [],
                                                    style={
                                                        'num_format': '{0:.0f}',
                                                        'title': 'font-weight: bold; color: rgb(153,153,153); font-size: 32pt;',
                                                        'subtitle': 'font-weight: bold; font-size: 12pt;',
                                                        'numtitle': 'font-weight: bold; color: rgb(153,153,153); font-size: 8pt;',
                                                        'width': 300,
                                                        'height': 150
                                                    }
                                                    ),
            'collector.job.logs.ERROR': NumberWidget(self, 'Problems', 'count', [],
                                                     style={
                                                         'num_format': '{0:.0f}',
                                                         'title': 'font-weight: bold; color: rgb(153,153,153); font-size: 32pt;',
                                                         'subtitle': 'font-weight: bold; font-size: 12pt;',
                                                         'numtitle': 'font-weight: bold; color: rgb(153,153,153); font-size: 8pt;',
                                                         'width': 300,
                                                         'height': 150
                                                     }
                                                     ),
            'parking.image_input.bytes_read': NumberWidget(self, 'Data read', 'sum', [],
                                                           style={
                                                               'num_format': '{0:.2f} MB',
                                                               'title': 'font-weight: bold; color: rgb(153,153,153); font-size: 22pt;',
                                                               'subtitle': 'font-weight: bold; font-size: 12pt;',
                                                               'numtitle': 'font-weight: bold; color: rgb(153,153,153); font-size: 8pt;',
                                                               'width': 350,
                                                               'height': 150,
                                                               'multi': 1024 * 1024
                                                           }
                                                           )

        }
        for x in small_number_stats:
            small_stats_numbers_layout.addWidget(small_number_stats[x])
            self.number_stats[x] = small_number_stats[x]

        small_stats_numbers_layout.addStretch(1)
        small_stats_numbers_widget.setLayout(small_stats_numbers_layout)
        layout.addWidget(small_stats_numbers_widget)

        layout.addStretch(1)
        # layout.setAlignment(Qt.AlignTop)
        self.setLayout(layout)
        self.prec = None
        self.vals = None
        self.job = None


    def setJob(self, job):
        self.job = job
        self.initialState()

    def initialState(self):
        for x in self.number_stats:
            widget = self.number_stats[x]
            widget.clear()

    def calcOccupied(self, all, prec, op):
        if not all or not prec:
            return
        occup = []
        for x in range(len(all[prec]['parking.checker.places_all'][op])):
            all_v = all[prec]['parking.checker.places_all'][op][x]
            free_v = all[prec]['parking.checker.places_free'][op][x]
            if not all_v or not free_v:
                occup.append(None)
            else:
                occup.append(all_v - free_v)
        all[prec]['calculated.occupied'] = {
            op: occup
        }

    def statsLoaded(self, all):
        if not all:
            return
        self.calcOccupied(all, self.prec, 'avg')
        for stat_name in sorted(all[self.prec]):
            if not stat_name in self.number_stats:
                continue
            widget = self.number_stats[stat_name]
            widget.setStat(all[self.prec][stat_name], self.prec)

    def loadStats(self, prec, vals):
        self.prec = prec
        self.vals = vals
        job = self.parent.getJob()
        if not job or not 'created' in job:
            return
        async = getAsync()
        async.loadJobStats(self.statsLoaded, job['id'], prec, vals)



class JobStatsWidget(QWidget):
    def __init__(self, parent):
        super(QWidget, self).__init__(parent)
        self.parent = parent
        self.layout = QVBoxLayout()

        scroll = QtGui.QScrollArea()
        stats_all = QWidget()
        stats_all_layout = QVBoxLayout()

        self.stats = {
            'parking.checker.places_free': (StatsWidget(self, 'Free places'), ['avg', 'min', 'max']),
            'parking.checker.places_all': (StatsWidget(self, 'All places count'), ['max']),
            'parking.checker.check_place.time': (StatsWidget(self, 'Place check time (s)'), ['avg', 'min', 'max']),
            'parking.checker.check_image.time': (
            StatsWidget(self, 'Whole image check time (s)'), ['avg', 'min', 'max']),
            # 'parking.notifier.mqtt_messages_sent': StatsWidget(self,'Počet zaslaných mqtt správ')
        }
        for x in self.stats:
            stats_all_layout.addWidget(self.stats[x][0])
        stats_all.setLayout(stats_all_layout)

        scroll.setWidget(stats_all)
        scroll.setWidgetResizable(True)
        # scroll.setFixedHeight(400)

        self.layout.addWidget(scroll)
        self.layout.setAlignment(Qt.AlignTop)
        self.setLayout(self.layout)

        self.palette = {
            'avg': (153, 255, 255),
            'count': (255, 255, 153),
            'min': (153, 255, 153),
            'max': (255, 153, 204),
            'sum': (153, 153, 255)
        }
        self.prec = None
        self.vals = None
        self.job = None

    def setJob(self, job):
        self.job = job
        self.initialState()

    def initialState(self):
        for x in self.stats:
            widget = self.stats[x][0]
            widget.clear()


    def convertTime(self, times):
        if not times:
            return []
        LOCAL_TIMEZONE = datetime.now(timezone.utc).astimezone().tzinfo
        rt = []
        for t in times:
            rt.append(time.mktime(datetime.strptime(t, "%Y-%m-%dT%H:%M:%S").replace(tzinfo=timezone.utc).astimezone(
                LOCAL_TIMEZONE).timetuple()))
        return rt

    def withoutNones(self, time_vals, vals):
        rt_vals = []
        rt_times = []
        for x in range(len(vals)):
            if not vals[x] is None:
                rt_vals.append(vals[x])
                rt_times.append(time_vals[x])
        return rt_times, rt_vals

    def statsLoaded(self, all):
        if not all:
            return
        for stat_name in sorted(all[self.prec]):
            if not stat_name in self.stats:
                continue
            widget = self.stats[stat_name][0]
            time = all[self.prec][stat_name]['time']
            time = self.convertTime(time)
            widget.setXRange(time[0], time[-1])

            # min = self.min( all[prec][stat_name]['min'])
            # max = self.min( all[prec][stat_name]['max'])
            # widget.setYRange(min, max)
            ops = self.stats[stat_name][1]
            for op in ops:
                x, y = self.withoutNones(time, all[self.prec][stat_name][op])
                widget.plot(x, y, op, self.palette[op])

    def loadStats(self, prec, vals):
        self.prec = prec
        self.vals = vals
        job = self.parent.getJob()
        if not job or not 'created' in job:
            return
        async = getAsync()
        async.loadJobStats(self.statsLoaded, job['id'], prec, vals)


import os
import tempfile
import uuid

from PyQt5.QtCore import QRect, QMargins

CONFIG_ARCHIVE = "archive"
MARGIN = 5
# SUBWINDOW_ADJUST = 100
MAIN_WINDOW_SIZE = QRect(0,0,1200,800)
SUB_WINDOW_SIZE = QRect(0,0,1200,800)
QMARGIN = QMargins(MARGIN, MARGIN, MARGIN, MARGIN)
STEPS_ORDER = [
    'parking_image_input', 'parking_checker', 'notifier', 'metrics_collector' ,'archiver'
]

this_dir, this_filename = os.path.split(__file__)


new_job_template = {
    "id": "undefined",
    "name": 'unsaved new job',
    "priority": 0,
    "autostart": True,
    "routes": [
        {
            "name": "default_sequence",
            "type_name": "sequence",
            "steps": [
            ]
        }
    ],
    "required_workers": [

    ],
    "required_data_support": [
        {
            "name": "parking_detection_data",
            "content_type": "application/json"
        }
    ]
}


def local_job_dir( job_id ):
    jdir = os.path.join(tempfile.gettempdir(), job_id)
    if not os.path.exists(jdir):
        os.makedirs(jdir,exist_ok=True)
    return jdir

def local_img_name(job_id):
    return os.path.join( local_job_dir(job_id), track_uuid() + '.jpg' )


def get_job_step_configuration(job, job_step_type):
    if not job or not 'required_workers' in job:
        return None
    for wrk in job['required_workers']:
        if wrk['worker_type'] == job_step_type and 'configuration' in wrk:
            return wrk['configuration']
    return None

def get_job_step_configuration_value(job, job_step_type, conf_entry):
    conf = get_job_step_configuration(job,job_step_type)
    if conf and conf_entry in conf:
        return conf[conf_entry]
    else:
        return None

def track_uuid():
    return str(uuid.uuid1())


#!/usr/bin/env python

PROJECT = 'parking-app'

__VERSION__="0.0.4"

from setuptools import setup, find_packages

try:
    long_description = open('README.md', 'rt').read()
except IOError:
    long_description = ''

setup(
    name=PROJECT,
    version=__VERSION__,

    description='Parking desktop app',
    long_description=long_description,

    author='Richard Holly',
    author_email='rho@optimaideas.com',

    url='https://gitlab.com/optima_iot/parking_desktop_app',

    classifiers=['Programming Language :: Python :: 3',
                 'Programming Language :: Python :: 3.6',
                 'Intended Audience :: Developers',
                 'Environment :: Console',
                 ],

    platforms=['Any'],

    scripts=[],

    provides=[],
    install_requires=['PyQt5','imutils','pyqtgraph','opencv-python',
                      'requests','micropipes-cli==0.6', 'qdarkgraystyle','Pillow','matplotlib'],

    namespace_packages=[],
    packages=find_packages(),
    include_package_data=True,
    package_data = {
        'app': ['img/*'],
    },

    entry_points={
        'gui_scripts': [
            'parking-app = app.gui:main'
        ],
        'console_scripts': [
            'parking-app-debug = app.gui:main'
        ]
    },
    zip_safe=False,
)


from abc import abstractmethod
import json
from datetime import datetime

import cv2
import numpy as np
import requests
from PyQt5.QtCore import Qt, QTimer
from PyQt5.QtGui import QDoubleValidator
from PyQt5.QtWidgets import *
from requests.auth import *
import pyqtgraph as pg

from app.imagewidgets import JobImageWidget, JobImageFilesDialog
from app.utils import *
from app.borderlayout import *
import matplotlib.image as mpimg
import mimetypes
from app.async import AsyncComunicator, getAsync

pg.setConfigOptions(imageAxisOrder='row-major')

class Configuration(QMainWindow):

    def __init__(self, parent, title, width, height, job_step_type):
        super().__init__()
        self.parent = parent
        self.setWindowModality(Qt.ApplicationModal)
        self.title = title
        self.setWindowTitle(self.title.format(''))
        self.left = 0
        self.top = 0
        self.width = width
        self.height = height
        self.setGeometry(self.left, self.top, self.width, self.height)
        qtRectangle = self.frameGeometry()
        centerPoint = QDesktopWidget().availableGeometry().center()
        # centerPoint = mainWindow().geometry().center()
        qtRectangle.moveCenter(centerPoint)
        self.move(qtRectangle.topLeft())
        self.main_widget = QWidget()
        self.job_step_type = job_step_type
        self.job = None
        top_tabs = QTabWidget()
        top_tabs.addTab(self.main_widget, 'Configuration')
        self.raw = QTextEdit(parent)
        self.raw.setReadOnly(True)
        self.raw.setLineWrapMode(QTextEdit.NoWrap)
        top_tabs.addTab(self.raw, "Raw")
        top_tabs.currentChanged.connect(self.onTabChange)

        self.setCentralWidget(top_tabs)
        self.setContentsMargins(QMARGIN)

    def getConfiguration(self):
        if not self.job:
            return None
        if not 'required_workers' in self.job:
            self.job['required_workers'] = [
                {
                    "worker_type": self.job_step_type,
                    "configuration": {
                    }
                }
            ]
        worker = None
        for wrk in self.job['required_workers']:
            if wrk['worker_type'] == self.job_step_type:
                worker = wrk
                break
        if not worker:
            worker = {
                "worker_type": self.job_step_type,
                "configuration": {
                }
            }
            self.job['required_workers'].append(worker)
        return worker['configuration']

    def removeConfiguration(self):
        if not self.job:
            return
        if not 'required_workers' in self.job:
            return
        found = None
        for wrk in self.job['required_workers']:
            if wrk['worker_type'] == self.job_step_type:
                found = wrk
                break
        if found:
            self.job['required_workers'].remove(found)

    def configure(self, job):
        self.job = job
        self.setWindowTitle(self.title.format(job['name']))

    def ensureGet(self, cfg, varname, editname, default_val):
        if not varname in cfg:
            cfg[varname] = default_val
        editname.setText(str(cfg[varname]))

    def ensureSet(self, cfg, varname, editname, is_float = False):
        if is_float:
            cfg[varname] = float(editname.text())
        else:
            cfg[varname]  = editname.text()

    @abstractmethod
    def saveConfiguration(self):
        pass

    def onTabChange(self, int_t):
        if int_t > 0:
            self.saveConfiguration()
            cfg = self.getConfiguration()
            self.raw.clear()
            self.raw.insertPlainText(json.dumps(cfg, indent=4, sort_keys=True))


class ConfigurationArchive(Configuration):

    def __init__(self, parent, job_step_type):
        super().__init__(parent, 'Archive configuration - {}', 400, 200, job_step_type)
        self.parent = parent

        layout = QVBoxLayout()
        self.archiveStrategy = QComboBox()
        self.archiveStrategy.addItem('all')
        self.archiveStrategy.addItem('delete')
        self.archiveStrategy.addItem('changes')
        layout.addWidget(self.archiveStrategy)
        desc = QLabel()
        desc.setText('Archive strategy - what to do with data in every cycle\nall - keep all on disk (for 1 day)\n'
                     'delete - always delete (just send notifications)\nchanges - keep only changes for 1 day\n')
        layout.addWidget(desc)
        layout.addStretch(1)
        self.main_widget.setLayout(layout)

    def configure(self, job):
        super().configure(job)
        cfg = self.getConfiguration()
        if cfg != None:
            if not 'archive_strategy' in cfg:
                cfg['archive_strategy'] = 'changes'
            self.archiveStrategy.setCurrentText(cfg['archive_strategy'])

    def closeEvent(self, event):
        pass

    def saveConfiguration(self):
        text = self.archiveStrategy.currentText()
        cfg = self.getConfiguration()
        cfg['archive_strategy'] = text

class ConfigurationImageInput(Configuration):

    def __init__(self, parent, job_step_type):
        super().__init__(parent, 'Camera input - {}', 1200, 500, job_step_type)
        self.parent = parent

        top_layout = QHBoxLayout()
        self.image_widget = JobImageWidget(self)
        top_layout.addWidget(self.image_widget)

        url_label = QLabel('Url')
        user_label = QLabel('Username')
        pass_label = QLabel('Password')
        delay_label = QLabel('Delay (sec.)')

        self.url_edit = QLineEdit()
        self.user_edit = QLineEdit()
        self.pass_edit = QLineEdit()
        self.pass_edit.setEchoMode(QLineEdit.Password)
        self.delay_edit = QLineEdit()
        delay_validator = QDoubleValidator(0.001, 3600, 3)
        self.delay_edit.setValidator(delay_validator)

        stretch_widget = QWidget()
        stretch = QVBoxLayout()

        attribs_widget = QWidget()
        grid = QGridLayout()
        grid.setSpacing(MARGIN)
        etuples = [
            (url_label, self.url_edit),
            (user_label, self.user_edit),
            (pass_label, self.pass_edit),
            (delay_label, self.delay_edit),
        ]
        for x in range(len(etuples)):
            grid.addWidget(etuples[x][0], x + 1, 0)
            grid.addWidget(etuples[x][1], x + 1, 1)

        attribs_widget.setLayout(grid)
        stretch.addWidget(attribs_widget)
        stretch.addStretch(1)
        self.status_messages = QTextEdit(parent)
        self.status_messages.setReadOnly(True)
        self.status_messages.setLineWrapMode(QTextEdit.NoWrap)
        stretch.addWidget(self.status_messages)
        buttons_widget = QWidget()
        buttons_layout = QHBoxLayout()
        self.test_button = QPushButton('Start test')
        self.test_button.setCheckable(True)
        self.test_button.clicked.connect(self.test_cam)
        buttons_layout.addWidget(self.test_button)
        buttons_layout.addStretch(1)
        # set_map_button = QPushButton('Set as map')
        # set_map_button.clicked.connect(self.set_map)
        # buttons_layout.addWidget(set_map_button)
        buttons_widget.setLayout(buttons_layout)

        stretch.addWidget(buttons_widget)
        stretch_widget.setLayout(stretch)

        top_layout.addWidget(stretch_widget)
        self.main_widget.setLayout(top_layout)

        self.communicator = AsyncComunicator(None, self.status_messages)

        self.timer = QTimer()
        self.timer.setSingleShot(False)
        self.timer.timeout.connect(self.test_cam_reload)
        self.timer_stop()


    def configure(self, job):
        if self.job and self.job['id'] != job['id']:
            self.status_messages.clear()
            self.image_widget.clearImage()
        super().configure(job)
        cfg = self.getConfiguration()
        if cfg != None:
            self.ensureGet(cfg, 'img_url', self.url_edit, 'https://')
            self.ensureGet(cfg, 'img_url_username', self.user_edit, '')
            self.ensureGet(cfg, 'img_url_pass', self.pass_edit, '')
            self.ensureGet(cfg, 'delay_sec', self.delay_edit, 30)

    def timer_stop(self):
        self.timer.stop()
        self.test_button.setText('Start test')
        self.test_button.setStyleSheet("background-color:rgb(153,255,153); color:rgb(0,0,0);")
        self.test_button.setIcon(QApplication.style().standardIcon(QStyle.SP_MediaPlay))

    def timer_start(self):
        self.timer.start(1000)
        self.test_button.setText('Stop test')
        self.test_button.setStyleSheet("background-color:rgb(255,153,153); color:rgb(0,0,0);")
        self.test_button.setIcon(QApplication.style().standardIcon(QStyle.SP_MediaStop))

    def closeEvent(self, event):
        self.timer_stop()
        self.saveConfiguration()

    def saveConfiguration(self):
        cfg = self.getConfiguration()
        if cfg != None:
            self.ensureSet(cfg, 'img_url', self.url_edit)
            self.ensureSet(cfg, 'img_url_username', self.user_edit)
            self.ensureSet(cfg, 'img_url_pass', self.pass_edit)
            self.ensureSet(cfg, 'delay_sec', self.delay_edit, True)

    def test_cam(self, start):
        if start:
            self.timer_start()
        else:
            self.timer_stop()

    # def grab_data(self):
    #     try:
    #         img_url = self.url_edit.text()
    #         img_url_user = self.user_edit.text()
    #         img_url_pass = self.pass_edit.text()
    #         self.status_messages.append('{} Connecting to {} as {} with {}'.format(str(datetime.now().isoformat()), img_url, img_url_user, img_url_pass))
    #         response = requests.get(img_url, auth=HTTPDigestAuth(img_url_user, img_url_pass), verify=False)
    #         self.status_messages.append('{} Response status is {} - {}'.format(str(datetime.now().isoformat()), response.status_code, response.reason))
    #         if response.status_code == 200:
    #             btra = bytearray(response.content)
    #             return cv2.imdecode(np.asarray(btra), 1), len(btra)
    #     except Exception as e:
    #         self.status_messages.append(
    #             '{} error {}'.format(str(datetime.now().isoformat()), e))
    #     return None, 0

    def test_data_loaded(self, result_tuple):
        img, bytes_count = result_tuple
        if bytes_count > 0:
            img_url = self.url_edit.text()
            self.image_widget.setImage(img, img_url, str(datetime.now().isoformat()))
            self.save_cv_img(img)

    def save_cv_img(self, cv_img):
        try:
            tmpname = local_img_name(self.job['id'])
            cv2.imwrite(tmpname, cv_img)
        except Exception as e:
            print(e)

    def test_cam_reload(self):
        self.communicator.grabData(self.test_data_loaded, self.url_edit.text(), self.user_edit.text(), self.pass_edit.text() )




    # def set_map(self):
    #     self.parent.setTemporaryImageMap(self.image_widget.get_cv_img())

class ConfigurationRegions(Configuration):

    def __init__(self, parent, job_step_type):
        super().__init__(parent, 'Regions configuration - {}', 1200, 800, job_step_type)
        self.parent = parent

        layout = BorderLayout()
        selected_region_widget = QWidget()
        selected_region_layout = QGridLayout()
        selected_region_layout.addWidget(QLabel('Selected area'), 1,0)
        self.selected_area_name = QLineEdit()
        selected_region_layout.addWidget(self.selected_area_name, 1,1)
        selected_region_layout.addWidget(QLabel('Coordinates'), 2,0)
        self.selected_area_coords = QLabel()
        selected_region_layout.addWidget(self.selected_area_coords, 2,1)

        buttons = QWidget()
        set_background_button = QPushButton("Select background map")
        set_background_button.clicked.connect(self.set_background_map)
        delete_button = QPushButton("Delete region")
        delete_button.clicked.connect(self.roiDelete)
        edit_button = QPushButton("Save name")
        edit_button.clicked.connect(self.roiNameChanged)
        hbox = QHBoxLayout()
        hbox.addWidget(edit_button)
        hbox.addWidget(delete_button)
        hbox.addStretch(1)
        hbox.addWidget(set_background_button)
        vbox = QVBoxLayout()
        vbox.addStretch(1)
        vbox.addLayout(hbox)
        buttons.setLayout(vbox)
        selected_region_layout.addWidget(buttons,3, 1)
        selected_region_widget.setLayout(selected_region_layout)
        layout.addWidget(selected_region_widget, BorderLayout.North)
        self.gv = pg.GraphicsLayoutWidget(parent=self)
        self.gv_view = self.gv.addPlot(lockAspect=True)
        self.gv_view.invertY()

        layout.addWidget(self.gv, BorderLayout.Center)
        self.main_widget.setLayout(layout)
        self.blue_pen = pg.mkPen(color='b', width=3)
        self.counter = 0
        self.rois = []
        self.rois_texts = []
        self.region_selected = None



    def configure(self, job):
        super().configure(job)
        cfg = self.getConfiguration()
        self.gv_view.clear()
        self.rois.clear()
        self.rois_texts.clear()
        self.counter = 0
        if cfg != None:
            self.ima = pg.ImageItem()
            self.ima.mouseClickEvent = self.mouseClickArea
            self.gv_view.addItem(self.ima)
            if 'map' in cfg:
                jdir = local_job_dir(self.job['id'])
                ffname = os.path.join(jdir, cfg['map'])
                if not os.path.exists(ffname):
                    self.downloadUsedMap(cfg['map'])
                else:
                    self.setUsedMap(ffname)

            if not 'regions' in cfg:
                cfg['regions'] = []
            self.regions = cfg['regions']
            if not self.regions:
                return
            for item in self.regions:
                roi = pg.PolyLineROI([], pen=self.blue_pen, closed=True, removable=True)
                roi.setState(item['roi'])
                roi.setAcceptedMouseButtons(Qt.LeftButton)
                roi.sigRemoveRequested.connect(self.removeRoi)
                roi.sigRegionChangeFinished.connect(self.roiChanged)
                roi.sigClicked.connect(self.roiClicked)
                self.gv_view.addItem(roi)
                text = pg.TextItem()
                # text.setAnchor(-10)
                text.setText(str(item['name']), color='w')
                text.setParentItem(roi)
                rect = roi.boundingRect()
                text.setPos(rect.x(), rect.y())
                self.rois.append(roi)
                self.rois_texts.append(text)
                self.counter += 1


    def closeEvent(self, event):
        # transform points - adjust pos to 0,0
        cfg = self.getConfiguration()
        self.regions = cfg['regions']
        if not self.regions:
            return
        for item in self.regions:
            pos0, pos1 = item['roi']['pos']
            if pos0 == 0 and pos1 == 0:
                continue
            points = item['roi']['points']
            points2 = []
            for p0, p1 in points:
                p0 += pos0
                p1 += pos1
                points2.append((p0,p1))
            item['roi']['pos'] = (0,0)
            item['roi']['points'] = points2


    def saveConfiguration(self):
        pass


    def mouseClickArea(self, event):
        mouse_x = round(event.pos().x())
        mouse_y = round(event.pos().y())
        if event.double():
            # print( 'Click at {} {} '.format(mouse_x,mouse_y))
            pen = pg.mkPen(color='b', width=3)
            roi = pg.PolyLineROI(
                [[mouse_x, mouse_y], [mouse_x + 40, mouse_y], [mouse_x + 40, mouse_y + 40], [mouse_x, mouse_y + 40]],
                pen=pen, closed=True, removable=True)
            roi.setAcceptedMouseButtons(Qt.LeftButton)
            roi.sigRemoveRequested.connect(self.removeRoi)
            roi.sigRegionChangeFinished.connect(self.roiChanged)
            roi.sigClicked.connect(self.roiClicked)
            self.gv_view.addItem(roi)
            #
            name = 'place_{}'.format(self.counter+1)
            text = pg.TextItem()
            # # text.setAnchor(-10)
            text.setText(name, color='w')
            text.setParentItem(roi)
            text.setPos(mouse_x, mouse_y)
            #
            rois_data_item = {
                'name':name,
                'roi': roi.saveState()
                # ,'rect': self.rect(roi)
            }

            self.regions.append(rois_data_item)
            self.rois.append(roi)
            self.rois_texts.append(text)
            #
            # self.table.cellChanged.disconnect(self.tableChanged)
            # self.table.setData(self.regions)
            # self.table.cellChanged.connect(self.tableChanged)
            self.counter += 1

    def removeRoi(self, roi):
        idx = self.rois.index(roi)
        self.rois.pop(idx)
        self.regions.pop(idx)
        self.rois_texts.pop(idx)
        self.gv_view.removeItem(roi)
        return True


    def roiChanged(self, roi):
        idx = self.rois.index(roi)
        self.regions[idx]['roi'] = roi.saveState()

    def roiClicked(self, roi):
        # print(roi)
        idx = self.rois.index(roi)
        if self.region_selected and self.region_selected in self.regions:
            pidx = self.regions.index(self.region_selected)
            self.rois_texts[pidx].setColor('w')
        self.region_selected = self.regions[idx]
        self.selected_area_name.setText(str(self.region_selected['name']))
        self.selected_area_coords.setText(str(self.region_selected['roi']))
        self.rois_texts[idx].setColor('y')

    def roiNameChanged(self):
        if not self.region_selected or not self.region_selected in self.regions:
            return
        idx = self.regions.index(self.region_selected)
        self.regions[idx]['name'] = self.selected_area_name.text()
        self.rois_texts[idx].setText(self.selected_area_name.text(), color='w')

    def roiDelete(self):
        if not self.region_selected or not self.region_selected in self.regions:
            return
        idx = self.regions.index(self.region_selected)
        roi = self.rois[idx]
        self.rois.pop(idx)
        self.regions.pop(idx)
        self.rois_texts.pop(idx)
        self.gv_view.removeItem(roi)

    def set_background_map(self):
        dialog = JobImageFilesDialog(self, self.job['id'])
        dialog.exec_()
        if dialog.selected_file:
            cfg = self.getConfiguration()
            if cfg:
                cfg['map'] = dialog.selected_file
                self.configure(self.job)


    def setUsedMap(self, ffname):
        if ffname and os.path.exists(ffname):
            self.map_data = mpimg.imread(ffname)
            self.ima.setImage(self.map_data)

    def downloadUsedMap(self, mapfile):
        async = getAsync()
        async.download_used_map(self.setUsedMap, self.job['id'], mapfile)


class ConfigurationMqttNotifier(Configuration):

    def __init__(self, parent, job_step_type):
        super().__init__(parent, 'Mqtt notifier - {}', 800, 500, job_step_type)
        self.parent = parent

        mqtt_camera_name_label = QLabel('mqtt_camera_name')
        mqtt_user_name_label = QLabel('mqtt_user_name')
        mqtt_api_key_label = QLabel('mqtt_api_key')
        mqtt_url_label = QLabel(' mqtt_url')
        mqtt_device_model_label = QLabel('mqtt_device_model')
        mqtt_urn_prefix_label = QLabel('mqtt_urn_prefix')
        mqtt_data_topic_label = QLabel('mqtt_data_topic')

        self.mqtt_camera_name_edit = QLineEdit()
        self.mqtt_user_name_edit = QLineEdit()
        self.mqtt_api_key_edit =QLineEdit()
        self.mqtt_url_edit =QLineEdit()
        self.mqtt_device_model_edit = QLineEdit()
        self.mqtt_urn_prefix_edit = QLineEdit()
        self.mqtt_data_topic_edit = QLineEdit()

        grid = QGridLayout()
        grid.setSpacing(MARGIN)
        etuples = [
            (mqtt_camera_name_label, self.mqtt_camera_name_edit),
            (mqtt_user_name_label,  self.mqtt_user_name_edit),
            (mqtt_api_key_label, self.mqtt_api_key_edit),
            (mqtt_url_label, self.mqtt_url_edit),
            (mqtt_device_model_label, self.mqtt_device_model_edit),
            (mqtt_urn_prefix_label, self.mqtt_urn_prefix_edit),
            (mqtt_data_topic_label, self.mqtt_data_topic_edit),
        ]
        for x in range(len(etuples)):
            grid.addWidget(etuples[x][0], x + 1, 0)
            grid.addWidget(etuples[x][1], x + 1, 1)

        self.main_widget.setLayout(grid)



    def configure(self, job):
        super().configure(job)
        cfg = self.getConfiguration()
        if cfg != None:
            self.ensureGet(cfg, 'mqtt_camera_name', self.mqtt_camera_name_edit, 'camera:__location__')
            self.ensureGet(cfg, 'mqtt_user_name', self.mqtt_user_name_edit, 'json+device')
            self.ensureGet(cfg, 'mqtt_api_key', self.mqtt_api_key_edit, '')
            self.ensureGet(cfg, 'mqtt_url', self.mqtt_url_edit, 'liveobjects.orange-business.com')
            self.ensureGet(cfg, 'mqtt_device_model', self.mqtt_device_model_edit, 'parkingCameraSpot_v1_0')
            self.ensureGet(cfg, 'mqtt_urn_prefix', self.mqtt_urn_prefix_edit, 'urn:lo:nsid:mqtt:')
            self.ensureGet(cfg, 'mqtt_data_topic', self.mqtt_data_topic_edit, 'dev/data')


    def closeEvent(self, event):
        self.saveConfiguration()

    def saveConfiguration(self):
        cfg = self.getConfiguration()
        if cfg != None:
            self.ensureSet(cfg, 'mqtt_camera_name', self.mqtt_camera_name_edit)
            self.ensureSet(cfg, 'mqtt_user_name', self.mqtt_user_name_edit)
            self.ensureSet(cfg, 'mqtt_api_key', self.mqtt_api_key_edit)
            self.ensureSet(cfg, 'mqtt_url', self.mqtt_url_edit)
            self.ensureSet(cfg, 'mqtt_device_model', self.mqtt_device_model_edit)
            self.ensureSet(cfg, 'mqtt_urn_prefix', self.mqtt_urn_prefix_edit)
            self.ensureSet(cfg, 'mqtt_data_topic', self.mqtt_data_topic_edit)


class ConfigurationMetricsCollector(Configuration):

    def __init__(self, parent, job_step_type):
        super().__init__(parent, 'Metrics collector configuration - {}', 400, 200, job_step_type)
        self.parent = parent

        layout = QVBoxLayout()
        # self.archiveStrategy = QComboBox()
        # self.archiveStrategy.addItem('all')
        # self.archiveStrategy.addItem('delete')
        # self.archiveStrategy.addItem('changes')
        # layout.addWidget(self.archiveStrategy)
        desc = QLabel()
        desc.setText('Collect and save metrics')
        layout.addWidget(desc)
        layout.addStretch(1)
        self.main_widget.setLayout(layout)

    def configure(self, job):
        super().configure(job)

    def closeEvent(self, event):
        self.saveConfiguration()

    def saveConfiguration(self):
        cfg = self.getConfiguration()
        cfg['dummy'] = 'dummy'
